/**
 * @file          /kylin-license/src/notify/main.cpp
 * @brief
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#include <kiran-log/qt5-log-i.h>
#include <ks-license/license-i.h>
#include <locale.h>
#include <QCoreApplication>
#include <QLoggingCategory>
#include <QObject>
#include <QTranslator>
#include <iostream>
#include "cmake-define.h"
#include "notification-wrapper.h"
#include "notify-default.h"
#include "notify-hw-joint.h"
#include "notify-pg.h"

#define TRANSLATION_DIR TRANSLATIONS_FILE_DIR
#define RESTRICTION_HW_JOINT "hw_joint"
#define RESTRICTION_PG "pg"
#define RESTRICTION_DEFAULT "default"

int main(int argc, char* argv[])
{
    QCoreApplication a(argc, argv);

    /// 加载翻译文件
    QTranslator* qtTranslator = new QTranslator(qApp);
    if (qtTranslator->load(QLocale(), "ksl-notify", "_", TRANSLATION_DIR, ".qm"))
    {
        a.installTranslator(qtTranslator);
    }
    else
    {
        KLOG_DEBUG() << "Load Translator File failed : " << TRANSLATION_DIR;
    }

    QLoggingCategory::defaultCategory()->setEnabled(QtDebugMsg, true);
    if (klog_qt5_init("", "kylinsec-session", PROJECT_NAME, "ksl-notify") < 0)
    {
        std::cout << "init zlog error" << std::endl;
    }

    try
    {
        Notify::NotificationWrapper::globalInit(QObject::tr("KylinSec").toStdString());
        if (RESTRICTION_POLICY == RESTRICTION_HW_JOINT)
        {
            auto notify = new Notify::NotifyHWJoint();
        }
        else if (RESTRICTION_POLICY == RESTRICTION_PG)
        {
            auto notify = new Notify::NotifyPG();
        }
        else
        {
            auto notify = new Notify::NotifyDefault();
        }
    }
    catch (const std::exception& e)
    {
        KLOG_WARNING("%s", e.what());
    }

    a.exec();
    return 0;
}
