#ifndef NOTIFYBASE_H
#define NOTIFYBASE_H

#include <ks-license/license-i.h>
#include <QDateTime>
#include <QMap>
#include <QObject>
#include <QTimer>

namespace Notify
{
#define KIRAN_SCREENSAVER_DBUS_NAME "com.kylinsec.Kiran.ScreenSaver"
#define KIRAN_SCREENSAVER_DBUS_PATH "/com/kylinsec/Kiran/ScreenSaver"
#define KIRAN_SCREENSAVER_DBUS_INTERFACE "com.kylinsec.Kiran.ScreenSaver"

#define MATE_SCREENSAVER_DBUS_NAME "org.mate.ScreenSaver"
#define MATE_SCREENSAVER_DBUS_PATH "/"
#define MATE_SCREENSAVER_DBUS_INTERFACE "org.mate.ScreenSaver"

typedef struct NotifyType
{
    NotifyType() : loopNotify(false), loopSecs(0) {}
    QString notifyName;
    bool loopNotify;
    int loopSecs;
} NotifyType;

struct LicenseInfo
{
    LicenseInfo() : licensed(LicenseActivationStatus::LAS_LAST),
                    serviceStatus(LicenseServiceStatus::LICENSE_SERVICE_STATUS_LAST),
                    expiredTime(0),
                    activationType(LicenseActivationType::LICENSE_ACTIVATION_TYPE_UNKNOWN),
                    firstBootTime(0),
                    activationTime(0),
                    updateMode(LicenseUpdateMode::LICENSE_UPDATE_MODE_NONE),
                    serviceMode(LicenseServiceMode::LICENSE_SERVICE_MODE_NONE),
                    exceptionStatus(LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_NONE) {}
    // 激活状态
    qint32 licensed;
    // 服务状态
    qint32 serviceStatus;
    // 激活码
    QString activationCode;
    // 机器码
    QString machineCode;
    // 客户代码
    QString customerCode;
    // 过期时间
    time_t expiredTime;
    // 激活方式，如果为LICENSE_ACTIVATION_TYPE_UNKNOWN，则激活时间字段不可用
    qint32 activationType;
    // 第一次使用系统的时间
    time_t firstBootTime;
    // 激活时间
    time_t activationTime;
    // 激活服务器地址
    QString authServer;
    // 服务号
    QString serviceUid;
    // 更新模式
    qint32 updateMode;
    // 服务模式
    qint32 serviceMode;
    // 异常状态
    qint32 exceptionStatus;
};

class NotifyBase : public QObject
{
    Q_OBJECT
public:
    explicit NotifyBase(QObject *parent = NULL);
    // flagDay 正数代表过期前天数、负数代表过期后天数
    bool registerNotify(bool licenseState,
                        int64_t flagDay,
                        QString notifyName,
                        bool loopNotify = false,
                        int loopSecs = 0);

    void registerNormalNotify(bool licenseState,
                              QString notifyName,
                              bool loopNotify = false,
                              int loopSecs = 0);

    LicenseInfo getLicenseInfo() { return m_licenseInfo; }

    void run();

private:
    bool insertNotifyType(QMap<int64_t, NotifyType> &stateMap,
                          int64_t flagDay,
                          QString notifyName,
                          bool loopNotify = false,
                          int loopSecs = 0);
    LicenseInfo getLicense();
    void setLoopNotify(QString notifyName, int loopSecs);

private slots:
    void timingCheckLicense();
    void loopNotify();
    void onActiveChange(bool state);

public slots:
    virtual void notifyActive(qint64 remainDays, QString notifyName) = 0;

signals:
    void sigActiveNotify(qint64 remainDays, QString notifyName);

public:
    // 服务模式和更新模式，继承该类的子类可以直接使用该参数进行功能定制
    LicenseInfo m_licenseInfo;

private:
    QTimer *m_licenseCheckTimer;
    QTimer *m_loopNotifyTimer;
    QString m_currentNotifyName;
    qint64 m_remainLicenseDays;
    int m_checkLicenseTimes;

    QMap<int64_t, NotifyType> m_inactiveTypes;
    QMap<int64_t, NotifyType> m_activedTypes;
    NotifyType m_activedNormalType;
    NotifyType m_inactiveNormalType;
};
}  // namespace Notify

#endif  // NOTIFYBASE_H
