#include "notify-hw-joint.h"

#include <fcntl.h>
#include <json/json.h>
#include <kiran-log/qt5-log-i.h>
#include <sys/stat.h>
#include <QtDBus>
#include <memory>
#include "lib/dbus/license-dbus.h"
#include "notification-wrapper.h"

namespace Notify
{
#define KIRAN_SCREENSAVER_METHOD_LOCK_SCREEN "Lock"
#define MATE_SCREENSAVER_METHOD_LOCK_SCREEN "Lock"

#define DBUS_TIMEOUT_MS (5000)
#define SERIOUS_EXPIRY_LOCK_INTERVAL (10 * 60)

#define SERIOUS_EXPIRY_FLAG_DAY (-90)
#define APPORCH_EXPIRY_FLAG_DAY (90)
#define EXPIRY_FLAG_DAY (0)

#define NORMAL_ACTIVED_NOTIFY "NormalActived"
#define NORMAL_INACTIVE_NOTIFY "NormalInactive"

#define INACTIVE_APPORCH_EXPIRY_NOTIFY "InactiveApporchExpiryNotify"
#define INACIIVE_EXPIRED_NOTIFY "InactiveExpiredNotify"
#define INACTIVE_SERIOUS_EXPIRED_NOTIFY "InactiveSeriousExpiryNotify"

#define ACTIVED_APPORCH_EXPIRY_NOTIFY "ActivedApporchExpiryNotify"
#define ACTIVED_EXPIRED_NOTIFY "ActivedExpiredNotify"
#define ACTIVED_SERIOUS_EXPIRED_NOTIFY "ActivedSeriousExpiryNotify"

NotifyHWJoint::NotifyHWJoint(QObject* parent) : NotifyBase(parent)

{
    m_notifyTimes = 0;
    registerNormalNotify(true, NORMAL_ACTIVED_NOTIFY);
    registerNormalNotify(false, NORMAL_INACTIVE_NOTIFY);

    registerNotify(false,
                   APPORCH_EXPIRY_FLAG_DAY,
                   INACTIVE_APPORCH_EXPIRY_NOTIFY);
    registerNotify(false,
                   EXPIRY_FLAG_DAY,
                   INACIIVE_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);
    registerNotify(false,
                   SERIOUS_EXPIRY_FLAG_DAY,
                   INACTIVE_SERIOUS_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);

    registerNotify(true,
                   APPORCH_EXPIRY_FLAG_DAY,
                   ACTIVED_APPORCH_EXPIRY_NOTIFY);
    registerNotify(true,
                   EXPIRY_FLAG_DAY,
                   ACTIVED_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);
    registerNotify(true,
                   SERIOUS_EXPIRY_FLAG_DAY,
                   ACTIVED_SERIOUS_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);

    this->run();
}

NotifyHWJoint::~NotifyHWJoint()
{
}

void NotifyHWJoint::notifyActive(qint64 remainDays, QString notifyType)
{
    m_notifyTimes++;
    if (notifyType == NORMAL_ACTIVED_NOTIFY)
    {
        NOTIFY_INFO(tr("Welcome to KylinSec OS!\n").toUtf8())
    }
    else if (notifyType == NORMAL_INACTIVE_NOTIFY)
    {
        NOTIFY_WARN(
            tr("Welcome to use KylinSec OS! Your system is not activated. "
               "Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
               "Contact us: 400-012-6606\n")
                .toUtf8())
    }

    else if (notifyType == INACTIVE_APPORCH_EXPIRY_NOTIFY)
    {
        NOTIFY_WARN(
            (tr("Welcome to use KylinSec OS! Your system is not activated. Trial expiration remaing:") +
             QString::number(remainDays) +
             tr(" days. Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
                " Contact us: 400-012-6606\n"))
                .toUtf8())
    }
    else if (notifyType == INACIIVE_EXPIRED_NOTIFY)
    {
        NOTIFY_INFO(
            (tr("Welcome to use KylinSec OS! Your system is not activated.After ") +
             QString::number(remainDays - SERIOUS_EXPIRY_FLAG_DAY) +
             tr(" days, your use will be affected. Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
                "Contact us: 400-012-6606\n"))
                .toUtf8())
    }
    else if (notifyType == INACTIVE_SERIOUS_EXPIRED_NOTIFY)
    {
        NOTIFY_WARN(
            tr("Welcome to use KylinSec OS! Your system is not activated, your use will be affected. "
               "Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
               "Contact us: 400-012-6606\n")
                .toUtf8())
        if (m_notifyTimes != 1)
        {
            QTimer::singleShot(10000, this, SLOT(lockScreen()));
        }
    }
    else if (notifyType == ACTIVED_APPORCH_EXPIRY_NOTIFY)
    {
        auto licenseInfo = getLicenseInfo();
        if (licenseInfo.updateMode == LicenseUpdateMode::LICENSE_UPDATE_MODE_SUBSCRIBE)
        {
            NOTIFY_WARN(
                (tr("The warranty period of your license for KylinSec OS will expired, remaining: ") +
                 QString::number(remainDays) +
                 tr(" days. Please renew as soon as possible.\n"
                    "Contact us: 400-012-6606\n"))
                    .toUtf8())
        }
        else
        {
            if (m_notifyTimes == 1)
            {
                NOTIFY_INFO(tr("Welcome to KylinSec OS!\n").toUtf8())
            }
        }
    }
    else if (notifyType == ACTIVED_EXPIRED_NOTIFY)
    {
        auto licenseInfo = getLicenseInfo();
        if (licenseInfo.updateMode == LicenseUpdateMode::LICENSE_UPDATE_MODE_SUBSCRIBE)
        {
            NOTIFY_WARN(
                tr("The warranty period of your license for KylinSec OS has expired. "
                   "Please purchase the technical service again.\n"
                   "Contact us: 400-012-6606\n")
                    .toUtf8())
        }
        else
        {
            if (m_notifyTimes == 1)
            {
                NOTIFY_WARN(
                    tr("The warranty period of your license for KylinSec OS has expired. "
                       "Please purchase the technical service again.\n"
                       "Contact us: 400-012-6606\n")
                        .toUtf8())
            }
        }
    }
    else if (notifyType == ACTIVED_SERIOUS_EXPIRED_NOTIFY)
    {
        auto licenseInfo = getLicenseInfo();
        if (licenseInfo.updateMode == LicenseUpdateMode::LICENSE_UPDATE_MODE_SUBSCRIBE)
        {
            NOTIFY_WARN(
                tr("The warranty period of your license for KylinSec OS has expired, your use will be affected. "
                   "Please purchase the technical service again.\n"
                   "Contact us: 400-012-6606\n")
                    .toUtf8())
            if (m_notifyTimes != 1)
            {
                QTimer::singleShot(10000, this, SLOT(lockScreen()));
            }
        }
        else
        {
            if (m_notifyTimes == 1)
            {
                NOTIFY_WARN(
                    tr("The warranty period of your license for KylinSec OS has expired. "
                       "Please purchase the technical service again.\n"
                       "Contact us: 400-012-6606\n")
                        .toUtf8())
            }
        }
    }
}

void NotifyHWJoint::lockScreen()
{
    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(KIRAN_SCREENSAVER_DBUS_NAME,
                                                                KIRAN_SCREENSAVER_DBUS_PATH,
                                                                KIRAN_SCREENSAVER_DBUS_INTERFACE,
                                                                KIRAN_SCREENSAVER_METHOD_LOCK_SCREEN);
    auto connection = QDBusConnection::sessionBus();
    auto msgReply = connection.call(msgMethodCall,
                                    QDBus::Block,
                                    DBUS_TIMEOUT_MS);
    if (msgReply.type() == QDBusMessage::ErrorMessage)
    {
        // 调用kiran-screensaver失败则调用mate-screensaver
        KLOG_INFO() << msgReply.errorMessage();
        msgMethodCall = QDBusMessage::createMethodCall(MATE_SCREENSAVER_DBUS_NAME,
                                                       MATE_SCREENSAVER_DBUS_PATH,
                                                       MATE_SCREENSAVER_DBUS_INTERFACE,
                                                       MATE_SCREENSAVER_METHOD_LOCK_SCREEN);
        msgReply = connection.call(msgMethodCall,
                                   QDBus::Block,
                                   DBUS_TIMEOUT_MS);
    }
}
}  // namespace Notify
