#include "notify-pg.h"

#include <fcntl.h>
#include <json/json.h>
#include <kiran-log/qt5-log-i.h>
#include <sys/stat.h>
#include <QtDBus>
#include <memory>
#include "lib/dbus/license-dbus.h"
#include "notification-wrapper.h"

namespace Notify
{
#define KIRAN_SCREENSAVER_METHOD_LOCK_SCREEN "Lock"
#define MATE_SCREENSAVER_METHOD_LOCK_SCREEN "Lock"

#define DBUS_TIMEOUT_MS 5000
#define SERIOUS_EXPIRY_LOCK_INTERVAL (60 * 60)

#define SERIOUS_EXPIRY_FLAG_DAY (-30)
#define APPORCH_EXPIRY_FLAG_DAY (90)
#define EXPIRY_FLAG_DAY (0)

#define NORMAL_ACTIVED_NOTIFY "NormalActived"
#define NORMAL_INACTIVE_NOTIFY "NormalInactive"

#define INACTIVE_APPORCH_EXPIRY_NOTIFY "InactiveApporchExpiryNotify"
#define INACIIVE_EXPIRED_NOTIFY "InactiveExpiredNotify"
#define INACTIVE_SERIOUS_EXPIRED_NOTIFY "InactiveSeriousExpiryNotify"

#define ACTIVED_APPORCH_EXPIRY_NOTIFY "ActivedApporchExpiryNotify"
#define ACTIVED_EXPIRED_NOTIFY "ActivedExpiredNotify"
#define ACTIVED_SERIOUS_EXPIRED_NOTIFY "ActivedSeriousExpiryNotify"

NotifyPG::NotifyPG(QObject* parent) : NotifyBase(parent)

{
    m_notifyTimes = 0;
    registerNormalNotify(true, NORMAL_ACTIVED_NOTIFY);
    registerNormalNotify(false, NORMAL_INACTIVE_NOTIFY);

    registerNotify(false,
                   APPORCH_EXPIRY_FLAG_DAY,
                   INACTIVE_APPORCH_EXPIRY_NOTIFY);
    registerNotify(false,
                   EXPIRY_FLAG_DAY,
                   INACIIVE_EXPIRED_NOTIFY);
    registerNotify(false,
                   SERIOUS_EXPIRY_FLAG_DAY,
                   INACTIVE_SERIOUS_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);

    registerNotify(true,
                   APPORCH_EXPIRY_FLAG_DAY,
                   ACTIVED_APPORCH_EXPIRY_NOTIFY);
    registerNotify(true,
                   EXPIRY_FLAG_DAY,
                   ACTIVED_EXPIRED_NOTIFY);
    registerNotify(true,
                   SERIOUS_EXPIRY_FLAG_DAY,
                   ACTIVED_SERIOUS_EXPIRED_NOTIFY,
                   true,
                   SERIOUS_EXPIRY_LOCK_INTERVAL);

    this->run();
}

NotifyPG::~NotifyPG()
{
}

void NotifyPG::showNormalActived()
{
    NOTIFY_INFO(tr("Welcome to KylinSec OS!\n").toUtf8())
}

void NotifyPG::showNormalInactive()
{
    NOTIFY_WARN(
        tr("Welcome to use KylinSec OS! Your system is not activated. "
           "Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
           "Contact us: 400-012-6606\n")
            .toUtf8())
}

void NotifyPG::showInactiveApproch(qint64 remainDays)
{
    NOTIFY_WARN(
        (tr("Welcome to use KylinSec OS! Your system is not activated. Remaing:") +
         QString::number(remainDays) +
         tr(" days. The system will be restricted after the expiration, and the relevant losses will be borne by yourself."
            "Please activate your system to enjoy the first-class and professional technical support from KylinSec team.\n"
            " Contact us: 400-012-6606\n"))
            .toUtf8())
}

void NotifyPG::showInactiveExpired(qint64 remainDays)
{
    NOTIFY_INFO(
        (tr("Welcome to use KylinSec OS! The system has expired, "
            "and your use will be affected. After ") +
         QString::number(remainDays - SERIOUS_EXPIRY_FLAG_DAY) +
         tr(" days, more strict system usage restrictions will be imposed, "
            "Please activate your system as soon as possible.\n"
            "Contact us: 400-012-6606\n"))
            .toUtf8())
}

void NotifyPG::showInactiveSeriousExpiry()
{
    NOTIFY_WARN(
        tr("Welcome to use KylinSec OS! The system has expired. "
           "The use of the system has been further restricted, and relevant losses caused shall be borne by yourself, "
           "Please activate your system as soon as possible.\n"
           "Contact us: 400-012-6606\n")
            .toUtf8())
    if (m_notifyTimes != 1)
    {
        QTimer::singleShot(10000, this, SLOT(lockScreen()));
    }
}

void NotifyPG::showActivedApproch(qint64 remainDays)
{
    auto licenseInfo = getLicenseInfo();
    if (licenseInfo.updateMode == LICENSE_UPDATE_MODE_SUBSCRIBE)
    {
        NOTIFY_WARN(
            (tr("Welcome to use KylinSec OS! Your license for KylinSec OS will expired. Remaing:") +
             QString::number(remainDays) +
             tr(" days. The system will be restricted after the expiration, and the relevant losses will be borne by yourself. "
                "Please renew as soon as possible to enjoy the first-class and professional technical support from KylinSec team.\n"
                "Contact us: 400-012-6606\n"))
                .toUtf8())
    }
    else
    {
        NOTIFY_WARN(
            (tr("Welcome to use KylinSec OS! Your license for KylinSec OS will expired. Remaing:") +
             QString::number(remainDays) +
             tr(" days. No longer provide technical services after the expiration. "
                "Please renew as soon as possible to enjoy the first-class and professional technical support from KylinSec team.\n"
                "Contact us: 400-012-6606\n"))
                .toUtf8())
    }
}

void NotifyPG::showActivedExpired(qint64 remainDays)
{
    auto licenseInfo = getLicenseInfo();
    if (licenseInfo.updateMode == LICENSE_UPDATE_MODE_SUBSCRIBE)
    {
        NOTIFY_INFO(
            (tr("Welcome to use KylinSec OS! The system has expired, "
                "and your use will be affected. After ") +
             QString::number(remainDays - SERIOUS_EXPIRY_FLAG_DAY) +
             tr(" days, more strict system usage restrictions will be imposed,"
                "Please renew your system as soon as possible.\n"
                "Contact us: 400-012-6606\n"))
                .toUtf8())
    }
    else
    {
        NOTIFY_WARN(
            (tr("The KylinSecOS has expired, and the technical service has been stopped. "
                "Please purchase the original technical service again. \n"
                "Contact: 400-012-6606\n"))
                .toUtf8())
    }
}

void NotifyPG::showActivedSeriousExpiry()
{
    auto licenseInfo = getLicenseInfo();
    if (licenseInfo.updateMode == LICENSE_UPDATE_MODE_SUBSCRIBE)
    {
        NOTIFY_WARN(
            tr("Welcome to use KylinSec OS! The system has expired. "
               "The use of the system has been further restricted, and relevant losses caused shall be borne by yourself, "
               "Please renew your system as soon as possible.\n"
               "Contact us: 400-012-6606\n")
                .toUtf8())
        if (m_notifyTimes != 1)
        {
            QTimer::singleShot(10000, this, SLOT(lockScreen()));
        }
    }
    else
    {
        NOTIFY_WARN(
            (tr("The KylinSecOS has expired, and the technical service has been stopped. "
                "Please purchase the original technical service again. \n"
                "Contact: 400-012-6606\n"))
                .toUtf8())
    }
}

void NotifyPG::notifyActive(qint64 remainDays, QString notifyType)
{
    m_notifyTimes++;
    if (notifyType == NORMAL_ACTIVED_NOTIFY)
    {
        showNormalActived();
    }
    else if (notifyType == NORMAL_INACTIVE_NOTIFY)
    {
        showNormalInactive();
    }
    else if (notifyType == INACTIVE_APPORCH_EXPIRY_NOTIFY)
    {
        showInactiveApproch(remainDays);
    }
    else if (notifyType == INACIIVE_EXPIRED_NOTIFY)
    {
        showInactiveExpired(remainDays);
    }
    else if (notifyType == INACTIVE_SERIOUS_EXPIRED_NOTIFY)
    {
        showInactiveSeriousExpiry();
    }
    else if (notifyType == ACTIVED_APPORCH_EXPIRY_NOTIFY)
    {
        showActivedApproch(remainDays);
    }
    else if (notifyType == ACTIVED_EXPIRED_NOTIFY)
    {
        showActivedExpired(remainDays);
    }
    else if (notifyType == ACTIVED_SERIOUS_EXPIRED_NOTIFY)
    {
        showActivedSeriousExpiry();
    }
}

void NotifyPG::lockScreen()
{
    QDBusMessage msgMethodCall = QDBusMessage::createMethodCall(KIRAN_SCREENSAVER_DBUS_NAME,
                                                                KIRAN_SCREENSAVER_DBUS_PATH,
                                                                KIRAN_SCREENSAVER_DBUS_INTERFACE,
                                                                KIRAN_SCREENSAVER_METHOD_LOCK_SCREEN);
    auto connection = QDBusConnection::sessionBus();
    auto msgReply = connection.call(msgMethodCall,
                                    QDBus::Block,
                                    DBUS_TIMEOUT_MS);
    if (msgReply.type() == QDBusMessage::ErrorMessage)
    {
        // 调用kiran-screensaver失败则调用mate-screensaver
        KLOG_INFO() << msgReply.errorMessage();
        msgMethodCall = QDBusMessage::createMethodCall(MATE_SCREENSAVER_DBUS_NAME,
                                                       MATE_SCREENSAVER_DBUS_PATH,
                                                       MATE_SCREENSAVER_DBUS_INTERFACE,
                                                       MATE_SCREENSAVER_METHOD_LOCK_SCREEN);
        msgReply = connection.call(msgMethodCall,
                                   QDBus::Block,
                                   DBUS_TIMEOUT_MS);
    }
}

}  // namespace Notify
