#include "active-frame.h"
#include <QDesktopWidget>
#include <QGraphicsDropShadowEffect>
#include <QMouseEvent>
#include "ui_active-frame.h"

ActiveFrame::ActiveFrame(QWidget* parent) : QWidget(parent),
                                            m_bDrag(false),
                                            ui(new Ui::ActiveFrame)
{
    ui->setupUi(this);
    initUi();
}

ActiveFrame::~ActiveFrame()
{
    delete ui;
}

void ActiveFrame::initUi()
{
    setWindowFlags(Qt::FramelessWindowHint);
    setWindowIcon(QIcon(":/images/icons/ksl-os-24.png"));
    ui->icon->setPixmap(QPixmap(":/images/icons/ksl-os-24.png"));
    ui->icon->setScaledContents(true);
    setWindowModality(Qt::ApplicationModal);
    setAttribute(Qt::WA_TranslucentBackground, true);

    int screenNum = QApplication::desktop()->screenNumber(QCursor::pos());
    QRect screenGeometry = QApplication::desktop()->screenGeometry(screenNum);
    this->move(screenGeometry.x() + (screenGeometry.width() - this->width()) / 2,
               screenGeometry.y() + (screenGeometry.height() - this->height()) / 2);

    m_frameLayout = new QVBoxLayout(this);
    m_frameLayout->setMargin(0);
    ui->widgetFrame->setLayout(m_frameLayout);
}

void ActiveFrame::setWidget(QWidget* widget)
{
    m_frameLayout->addWidget(widget);
    connect(widget, SIGNAL(sigClose()), this, SLOT(on_exit_clicked()));
}
void ActiveFrame::setTitle(const QString& title)
{
    ui->title->setText(title);
    setWindowTitle(title);
}
void ActiveFrame::on_exit_clicked()
{
    close();
}

void ActiveFrame::on_minimize_clicked()
{
    showMinimized();
}

void ActiveFrame::mousePressEvent(QMouseEvent* event)
{
    if (event->pos().y() < 40 && event->button() == Qt::LeftButton)
    {
        m_bDrag = true;
        m_mouseStartPoint = event->globalPos();
        m_windowTopLeftPoint = this->frameGeometry().topLeft();
    }
}

void ActiveFrame::mouseMoveEvent(QMouseEvent* event)
{
    if (m_bDrag)
    {
        QPoint distance = event->globalPos() - m_mouseStartPoint;
        move(m_windowTopLeftPoint + distance);
    }
}

void ActiveFrame::mouseReleaseEvent(QMouseEvent* event)
{
    if (event->button() == Qt::LeftButton)
    {
        m_bDrag = false;
    }
}
