/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd.
 * kiran-cpanel-system is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 * Author:     yuanxing <yuanxing@kylinos.com.cn>
 */

#include "kiran-system-activation.h"
#include <kiran-log/qt5-log-i.h>
#include <QDesktopWidget>
#include <QJsonDocument>
#include <QJsonObject>
#include <QPainter>
#include <QVBoxLayout>
#include <QX11Info>
#include <iostream>
#include "license-information.h"
#include "single-application.h"
#include "ui_kiran-system-activation.h"

#ifdef DISABLE_KIRANWIDGETS
KiranSystemActivation::KiranSystemActivation(QWidget* parent) : QWidget(parent),
                                                                m_bDrag(false),
                                                                ui(new Ui::KiranSystemActivation)
{
    ui->setupUi(this);
    setWindowTitle(tr("kiran system activation"));
    setWindowIcon(QIcon(":/images/icons/ksl-os-24.png"));
    ui->icon->setPixmap(QPixmap(":/images/icons/ksl-os-24.png"));
    ui->icon->setScaledContents(true);
    ui->title->setText(tr("kiran system activation"));
    setAttribute(Qt::WA_TranslucentBackground, true);
    connect(dynamic_cast<SingleApplication*>(qApp), &SingleApplication::instanceStarted, this, &KiranSystemActivation::handleInstanceStarted);
#else
KiranSystemActivation::KiranSystemActivation(QWidget* parent) : KiranTitlebarWindow(parent),
                                                                ui(new Ui::KiranSystemActivation)
{
    ui->setupUi(getWindowContentWidget());
    setTitle(tr("kiran system activation"));
    setIcon(QIcon(":/images/icons/ksl-os-24.png"));
#endif
    initUI();
}

KiranSystemActivation::~KiranSystemActivation()
{
    delete ui;
}

QSize KiranSystemActivation::sizeHint() const
{
    /*根据系统分辨率设置窗口大小*/
    return QSize(634, 680);
}

void KiranSystemActivation::paintEvent(QPaintEvent* painEvent)
{
    QStyleOption opt;

    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);

    QWidget::paintEvent(painEvent);
}

void KiranSystemActivation::initUI()
{
    QVBoxLayout* mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(0);
    ui->scrollAreaWidgetContents->setLayout(mainLayout);
    setWindowFlags(Qt::FramelessWindowHint);

    LicenseInformation* licenseInformation = new LicenseInformation(this);
    mainLayout->addWidget(licenseInformation);
}

void KiranSystemActivation::on_exit_clicked()
{
    close();
}

void KiranSystemActivation::on_minimize_clicked()
{
    showMinimized();
}

void KiranSystemActivation::mousePressEvent(QMouseEvent* event)
{
    if (event->pos().y() < 40 && event->button() == Qt::LeftButton)
    {
        m_bDrag = true;
        mouseStartPoint = event->globalPos();
        windowTopLeftPoint = this->frameGeometry().topLeft();
    }
}

void KiranSystemActivation::mouseMoveEvent(QMouseEvent* event)
{
    if (m_bDrag)
    {
        QPoint distance = event->globalPos() - mouseStartPoint;
        this->move(windowTopLeftPoint + distance);
    }
}

void KiranSystemActivation::mouseReleaseEvent(QMouseEvent* event)
{
    if (event->button() == Qt::LeftButton)
    {
        m_bDrag = false;
    }
}

void KiranSystemActivation::handleInstanceStarted()
{
    /*
     *由于QXcbWindow::requestActivateWindow
     *之中对root窗口发送_NET_ACTIVE_WINDOW的事件之中的时间戳未更新,
     *导致窗口管理器接收时事件戳较为落后，未被正确处理
     *暂时处理办法，手动更新下X11时间，避免事件戳落后
     */

    if (windowState() & Qt::WindowMinimized)
    {
        setWindowState(windowState() & ~Qt::WindowMinimized);
    }

    QX11Info::setAppTime(QX11Info::getTimestamp());
    raise();
    activateWindow();
}
