#include "license-application.h"
#include <kiran-log/qt5-log-i.h>
#include <QFile>
#include <QMessageBox>
#include <QStyle>

LicenseApplication::LicenseApplication(int &argc, char **argv) : QApplication(argc, argv)
{
    QString styleName = style()->objectName();
    if (!styleName.contains("kiran", Qt::CaseInsensitive))
    {
        // 默认设置黑色背景
        setQssFileByWindowColor(QColor(0, 0, 0));
        return;
    }
    KLOG_DEBUG() << styleName << ": " << styleName.contains("kiran", Qt::CaseInsensitive);

    connect(this, SIGNAL(paletteChanged(const QPalette &)), this, SLOT(onPaletteChanged(const QPalette &)));
    onPaletteChanged(palette());
}

LicenseApplication::~LicenseApplication()
{
}

void LicenseApplication::onPaletteChanged(const QPalette &palette)
{
    auto windowColor = palette.window().color();
    setQssFileByWindowColor(windowColor);
    KLOG_WARNING() << "onPaletteChanged textColor:" << QString::number(windowColor.rgb(), 16);
}

void LicenseApplication::setQssFileByWindowColor(QColor windowColor)
{
    QFile file;
    if (windowColor.black() < 127.5)
    {
        file.setFileName(":/qss/ksl-os-light.qss");
        KLOG_DEBUG() << "Set light qss";
    }
    else
    {
        file.setFileName(":/qss/ksl-os.qss");
        KLOG_DEBUG() << "Set dark qss";
    }

    if (file.open(QFile::ReadOnly))
    {
        QString styleSheet = QLatin1String(file.readAll());

        setStyleSheet(styleSheet);
        file.close();
    }
    else
    {
        QMessageBox::warning(NULL, QObject::tr("warning"), QObject::tr("Open qss file failed"), QMessageBox::Ok);
    }
}
