/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd.
 * kiran-cpanel-system is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 * Author:     yuanxing <yuanxing@kylinos.com.cn>
 */

#include "license-information.h"
#include <kiran-log/qt5-log-i.h>
#include <ks-license/license-i.h>
#include <QDateTime>
#include <QDesktopWidget>
#include <QFontMetrics>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonValue>
#include <QPainter>
#include <QStyleOption>
#include <iostream>
#include "dbus/license-dbus.h"
#include "ui_license-information.h"

namespace KylinSec
{
};
using namespace KylinSec;

#define SYSTEM_LOGO "KylinSec OS"

LicenseInformation::LicenseInformation(QWidget *parent) : QWidget(parent),
                                                          m_activeGuide(NULL),
                                                          m_licenseInfoWidget(NULL),
                                                          m_activeFrame(NULL),
                                                          ui(new Ui::LicenseInformation)
{
    ui->setupUi(this);

    setLicenseInfo();
#ifdef CONTACT_INFO_WITH_HINT
// 转接号码为3.4-3-DESKTOP桌面版本带量采购使用
    ui->lab_contact_info->setText(tr("400-012-6606 and then press the number 4, followed by the number 3."));
#else
    ui->lab_contact_info->setText("400-012-6606");
#endif

    connect(ui->btn_status, SIGNAL(clicked()), this, SLOT(onBtnStatusClicked()));
}

LicenseInformation::~LicenseInformation()
{
    delete ui;
    if (m_activeGuide != NULL)
    {
        delete m_activeGuide;
    }
    if (m_licenseInfoWidget != NULL)
    {
        delete m_licenseInfoWidget;
    }
    if (m_activeFrame != NULL)
    {
        delete m_activeFrame;
    }
}
void LicenseInformation::hideExtraWidget()
{
    ui->widget_update_mode->hide();
    ui->widget_service_mode->hide();
    ui->widget_exception->hide();
}

bool LicenseInformation::getLicneseInfo()
{
    QString licenseJson;
    if (!InfoDbus::KylinLicense::getLicenseJson(licenseJson))
    {
        m_licenseInfo.machineCode = "Unknow";
        return false;
    }

    QJsonParseError jsonError;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(licenseJson.toLocal8Bit().data(), &jsonError);
    if (jsonDocument.isNull() || jsonError.error != QJsonParseError::NoError)
    {
        KLOG_DEBUG() << " please check the activation information string " << licenseJson.toLocal8Bit().data();
        return false;
    }
    if (jsonDocument.isObject())
    {
        QJsonObject obj = jsonDocument.object();
        if (obj.contains(LICENSE_JK_ACTIVATION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_STATUS);
            if (value.isDouble())
            {
                m_licenseInfo.licensed = value.toVariant().toInt();
                KLOG_INFO() << "activation status :" << m_licenseInfo.licensed;
            }
        }
        if (obj.contains(LICENSE_JK_ACTIVATION_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_CODE);
            if (value.isString())
            {
                m_licenseInfo.activationCode = value.toVariant().toString();
                KLOG_INFO() << "activation code :" << m_licenseInfo.activationCode;
            }
        }
        if (obj.contains(LICENSE_JK_MACHINE_CODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_MACHINE_CODE);
            if (value.isString())
            {
                m_licenseInfo.machineCode = value.toVariant().toString();
                KLOG_INFO() << "machine code :" << m_licenseInfo.machineCode;
            }
        }
        if (obj.contains(LICENSE_JK_EXPIRED_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXPIRED_TIME);
            if (value.isDouble())
            {
                m_licenseInfo.expiredTime = time_t(value.toVariant().toUInt());
                KLOG_INFO() << "expired time :" << m_licenseInfo.expiredTime;
            }
        }
        if (obj.contains(LICENSE_JK_FIRST_BOOT_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_FIRST_BOOT_TIME);
            if (value.isDouble())
            {
                m_licenseInfo.firstBootTime = value.toVariant().toInt();
                KLOG_INFO() << "first boot time :" << m_licenseInfo.firstBootTime;
            }
        }
        if (obj.contains(LICENSE_JK_SERVICE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_SERVICE_MODE);
            if (value.isDouble())
            {
                m_licenseInfo.serviceMode = value.toVariant().toInt();
                KLOG_INFO() << "service mode :" << m_licenseInfo.serviceMode;
            }
        }
        if (obj.contains(LICENSE_JK_UPDATE_MODE))
        {
            QJsonValue value = obj.take(LICENSE_JK_UPDATE_MODE);
            if (value.isDouble())
            {
                m_licenseInfo.updateMode = value.toVariant().toInt();
                KLOG_INFO() << "update mode :" << m_licenseInfo.updateMode;
            }
        }
        if (obj.contains(LICENSE_JK_EXCEPTION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXCEPTION_STATUS);
            if (value.isDouble())
            {
                m_licenseInfo.exceptionStatus = value.toVariant().toInt();
                KLOG_INFO() << "exception status:" << m_licenseInfo.exceptionStatus;
            }
        }
    }
    return true;
}

void LicenseInformation::setErrorState()
{
    ui->lab_expire_date_info->setText(tr("Unknow"));
    ui->lab_activation_status_info->setText(tr("Can't get activation information"));
    ui->lab_activation_status_info->setStyleSheet("QLabel#lab_activation_status_info {color:#ff3838;font-family: \"Noto Sans CJK SC regular\";}");
    ui->lab_install_time_info->setText(tr("Unknow"));
    ui->btn_status->hide();
}

void LicenseInformation::setUnactivatedState()
{
    ui->btn_status->setText(tr("Activate"));

    QDateTime time = QDateTime::fromTime_t(m_licenseInfo.expiredTime);
    QString dueTime = time.toString("yyyy-MM-dd");

    ui->lab_activation_status_info->setText(QString("%1%2").arg(tr("Not activated. Trail expiration: ")).arg(dueTime));
    ui->lab_activation_status_info->setStyleSheet("QLabel#lab_activation_status_info {color:#ff3838;font-family: \"Noto Sans CJK SC regular\";}");

    if (m_licenseInfo.exceptionStatus == LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_INVALID)
    {
        ui->lab_activation_status_info->setText((QString("%1 (%2)")).arg(tr("The current time is illegal")).arg(tr("Less than the installation time")));
    }
    // 异常处理
    auto exceptionState = getExceptionString(m_licenseInfo.exceptionStatus);
    if (!exceptionState.isEmpty())
    {
        ui->widget_exception->show();
        ui->lab_exception_info->setText(getExceptionString(m_licenseInfo.exceptionStatus));
        ui->lab_exception_info->setStyleSheet("QLabel#lab_exception_info {color:#ff3838;font-family: \"Noto Sans CJK SC regular\";}");
    }

    ui->lab_expire_date_info->setText(tr("Not yet"));
}

void LicenseInformation::setActivatedState()
{
    ui->lab_expire_date_info->setText(QDateTime::fromTime_t(m_licenseInfo.expiredTime).toString("yyyy-MM-dd"));
    // 激活已过期，将激活按钮放出
    if (time(NULL) > m_licenseInfo.expiredTime)
    {
        ui->btn_status->setText(tr("Activate"));
        ui->lab_activation_status_info->setText(QString("%1").arg(tr("Activation has expired, please reactivate")));
        ui->lab_activation_status_info->setStyleSheet("QLabel#lab_activation_status_info {color:#ff3838;font-family: \"Noto Sans CJK SC regular\";}");
        return;
    }

    ui->lab_activation_status_info->setText(tr("Activated"));
    ui->lab_activation_status_info->setStyleSheet("QLabel#lab_activation_status_info {color:#5ab940;font-family: \"Noto Sans CJK SC regular\";}");

    ui->btn_status->setFixedSize(QSize(16, 16));
    ui->btn_status->setText(NULL);
    ui->btn_status->setIcon(QIcon(":/images/license_info.svg"));
    ui->btn_status->setStyleSheet("QToolButton#btn_status{background: transparent;}");

    QDateTime foreverTime = QDateTime::fromString("2100-01-01 0:0:0", "yyyy-MM-dd hh:mm:ss");
    if (m_licenseInfo.expiredTime >= foreverTime.toTime_t())
        ui->lab_expire_date_info->setText(tr("Forever"));

    // 服务模式设置
    auto serviceMode = getServiceModeString(m_licenseInfo.serviceMode);
    if (!serviceMode.isEmpty())
    {
        ui->lab_service_mode_info->setText(serviceMode);
        ui->widget_service_mode->show();
    }

    // 更新模式设置
    auto updateMode = getUpdateModeString(m_licenseInfo.updateMode);
    if (!updateMode.isEmpty())
    {
        ui->lab_update_mode_info->setText(updateMode);
        ui->widget_update_mode->show();
    }

    // 异常处理
    auto exception = getExceptionString(m_licenseInfo.exceptionStatus);
    if (!exception.isEmpty())
    {
        ui->lab_exception_info->setText(exception);
        ui->widget_exception->show();
        ui->lab_exception_info->setStyleSheet("QLabel#lab_exception_info {color:#ff3838;font-family: \"Noto Sans CJK SC regular\";}");
    }
}

/**
 * @brief SystemInformationWidget::setLicenseInfo:设置license信息
 */
void LicenseInformation::setLicenseInfo()
{
    // 收起动态加载的控件
    hideExtraWidget();

    if (!getLicneseInfo())
    {
        KLOG_DEBUG() << "get activation information failed";
        setErrorState();
        return;
    }

    /// 在界面上显示安装时间
    QString installTime = QDateTime::fromTime_t(m_licenseInfo.firstBootTime).toString("yyyy-MM-dd");
    ui->lab_install_time_info->setText(installTime);

    switch (m_licenseInfo.licensed)
    {
    case LicenseActivationStatus::LAS_UNACTIVATED:
    {
        setUnactivatedState();
        return;
    }
    case LicenseActivationStatus::LAS_ACTIVATED:
    {
        setActivatedState();
        break;
    }
    default:
        setErrorState();
        break;
    }
}

/**
 * @brief  槽函数，用户点击授权状态按钮，
 *         未激活，则弹出激活向导窗口
 *         已激活，则弹出授权信息窗口
 */
void LicenseInformation::onBtnStatusClicked()
{
    if (m_activeFrame != NULL)
    {
        delete m_activeFrame;
        m_activeFrame = NULL;
    }
    m_activeFrame = new ActiveFrame();

    if ((m_licenseInfo.licensed != LicenseStatus::LICENSE_STATUS_REGISTERD) ||
        m_licenseInfo.expiredTime < time(NULL))  // popup Active Guide
    {
        m_activeGuide = new ActGuideWidget(m_activeFrame);

        m_activeGuide->setAttribute(Qt::WA_QuitOnClose, false);
        m_activeGuide->installEventFilter(this);
        connect(m_activeGuide, SIGNAL(systemIsActived(bool)), this, SLOT(updateLicenseInfo(bool)));
        m_activeGuide->raise();
        m_activeFrame->setWidget(m_activeGuide);
        m_activeFrame->setTitle(tr("Activation Guide"));
    }
    else
    {
        m_licenseInfoWidget = new LicenseInfoWidget(m_licenseInfo.machineCode, m_licenseInfo.activationCode, m_activeFrame);
        m_licenseInfoWidget->setAttribute(Qt::WA_QuitOnClose, false);
        m_licenseInfoWidget->installEventFilter(this);
        m_licenseInfoWidget->setLicenseCode(m_licenseInfo.activationCode);
        m_licenseInfoWidget->setMachineCode(m_licenseInfo.machineCode);
        m_licenseInfoWidget->raise();

        m_activeFrame->setWidget(m_licenseInfoWidget);
        m_activeFrame->setTitle(tr("Activation Information"));
    }

    m_activeFrame->show();
}

/**
 * @brief  接收到激活向导窗口发出的激活状态信号，更新当前窗口的激活状态信息
 * @param  isregister 是否激活成功
 */
void LicenseInformation::updateLicenseInfo(bool isregister)
{
    KLOG_INFO() << "updateActivationInformation\n";
    if (isregister)
    {
        setLicenseInfo();
    }
}

bool LicenseInformation::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == m_activeGuide && event->type() == QEvent::Close)
    {
        m_activeGuide->deleteLater();
        m_activeGuide = NULL;
    }
    else if (obj == m_licenseInfoWidget && event->type() == QEvent::Close)
    {
        m_licenseInfoWidget->deleteLater();
        m_licenseInfoWidget = NULL;
    }
    return false;
}

void LicenseInformation::paintEvent(QPaintEvent *painEvent)
{
    QStyleOption opt;

    opt.init(this);
    QPainter painter(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter, this);
    QWidget::paintEvent(painEvent);

    QDate currentDate = QDate::currentDate();
    QString date = currentDate.toString("yyyy-MM-dd");
    QString year = date.left(4);
    QString copyright = QString(tr("Copyright ©")) + QString("%1 ").arg(year) + QString(tr("KylinSec. All rights reserved."));

    QFont font = QFont("Noto Sans CJK SC regular", 46);
    QRect drawRecLogo = QRect(this->geometry().x() + 24, this->geometry().y() + 16, this->width(), ui->widget_logo->height() - 16);

    painter.setPen(QColor(46, 179, 255));  // #2eb3FF
    painter.setFont(font);
    painter.drawText(drawRecLogo, SYSTEM_LOGO);

    QFontMetrics fm = painter.fontMetrics();
    int heightText = fm.height();

    int offsetHeight = heightText + 5 + 16;
    QRect drawRecCopyright = QRect(24, this->geometry().y() + offsetHeight, this->width(), ui->widget_logo->height() - offsetHeight);
    font.setPointSize(10);
    font.setWeight(QFont::Normal);
    painter.setPen(QColor(145, 145, 145));
    painter.setFont(font);
    painter.drawText(drawRecCopyright, copyright);
}

QSize LicenseInformation::sizeHint() const
{
    return QSize(600, 600);
}

QString LicenseInformation::getExceptionString(qint32 exceptionStatus)
{
    switch (exceptionStatus)
    {
    case LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_CS_TIME_DIFF_TOO_MUCH:
        return tr("The difference between the current time and the server time is too large, please check the system time");
    case LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_SERVER_NOT_CONNECTED:
        return tr("Communication failure with the server, please check the network");
    default:
        return QString();
    }
}

QString LicenseInformation::getUpdateModeString(qint32 updateMode)
{
    switch (updateMode)
    {
    case LicenseUpdateMode::LICENSE_UPDATE_MODE_PERMANENT:
        return tr("permanent");
    case LicenseUpdateMode::LICENSE_UPDATE_MODE_SUBSCRIBE:
        return tr("subscribe");
    default:
        return QString();
    }
}

QString LicenseInformation::getServiceModeString(qint32 serviceMode)
{
    switch (serviceMode)
    {
        // 7X24小时
    case LicenseServiceMode::LICENSE_SERVICE_MODE_7X24:
        return tr("7x24 hours");
        // 5X8小时
    case LicenseServiceMode::LICENSE_SERVICE_MODE_5X8:
        return tr("5x8 hours");
    default:
        return QString();
    }
}
