/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd.
 * kiran-cpanel-system is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 *
 * Author:     yuanxing <yuanxing@kylinos.com.cn>
 */

#ifndef LICENSEINFORMATION_H
#define LICENSEINFORMATION_H

#include <ks-license/license-i.h>
#include <QWidget>
#include "active-widgets/active-frame.h"
#include "active-widgets/active-guide-widget.h"
#include "active-widgets/license-info-widget.h"
namespace Ui
{
class LicenseInformation;
}

struct LicenseInfo
{
    LicenseInfo() : licensed(LicenseActivationStatus::LAS_LAST),
                    serviceStatus(LicenseServiceStatus::LICENSE_SERVICE_STATUS_LAST),
                    expiredTime(0),
                    activationType(LicenseActivationType::LICENSE_ACTIVATION_TYPE_UNKNOWN),
                    firstBootTime(0),
                    activationTime(0),
                    updateMode(LicenseUpdateMode::LICENSE_UPDATE_MODE_NONE),
                    serviceMode(LicenseServiceMode::LICENSE_SERVICE_MODE_NONE),
                    exceptionStatus(LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_NONE) {}
    // 激活状态
    qint32 licensed;
    // 服务状态
    qint32 serviceStatus;
    // 激活码
    QString activationCode;
    // 机器码
    QString machineCode;
    // 客户代码
    QString customerCode;
    // 过期时间
    time_t expiredTime;
    // 激活方式，如果为LICENSE_ACTIVATION_TYPE_UNKNOWN，则激活时间字段不可用
    qint32 activationType;
    // 第一次使用系统的时间
    time_t firstBootTime;
    // 激活时间
    time_t activationTime;
    // 激活服务器地址
    QString authServer;
    // 服务号
    QString serviceUid;
    // 更新模式
    qint32 updateMode;
    // 服务模式
    qint32 serviceMode;
    // 异常状态
    qint32 exceptionStatus;
};

class LicenseInformation : public QWidget
{
    Q_OBJECT

public:
    explicit LicenseInformation(QWidget *parent = NULL);
    ~LicenseInformation();
    virtual QSize sizeHint() const;

private:
    void hideExtraWidget();
    bool getLicneseInfo();
    void setLicenseInfo();
    void setErrorState();
    void setUnactivatedState();
    void setActivatedState();

    QString getExceptionString(qint32 exceptionStatus);
    QString getUpdateModeString(qint32 updateMode);
    QString getServiceModeString(qint32 updateMode);

public slots:
    void onBtnStatusClicked();
    void updateLicenseInfo(bool isregister);

protected:
    bool eventFilter(QObject *obj, QEvent *event);
    void paintEvent(QPaintEvent *);

private:
    Ui::LicenseInformation *ui;
    LicenseInfo m_licenseInfo;
    ActGuideWidget *m_activeGuide;
    LicenseInfoWidget *m_licenseInfoWidget;
    ActiveFrame *m_activeFrame;
};

#endif  // LICENSEINFORMATION_H
