#include "license-grpc.h"

LicenseGrpc::LicenseGrpc(QObject *parent) : QObject(parent)
{
    this->serverIP = "0.0.0.0";
}
bool LicenseGrpc::getPointsRecordsRsp(QList<Point> &pointsData)
{
    grpc::ClientContext context;
    ::PointsRecordsReq req;
    ::PointsRecordsRsp rsp;
    auto channel = grpc::CreateChannel(QString("%1:7788").arg(this->serverIP).toStdString(), grpc::InsecureChannelCredentials());
    auto stub = License::NewStub(channel);

    auto status = stub->GetPointsRecords(&context, req, &rsp);
    if (status.ok())
    {
        this->pointsUidList.clear();
        for (int i = 0; i < rsp.points_size(); i++)
        {
            Point pointData;
            auto point = rsp.points(i);
            pointData.uid = QString::fromStdString(point.uid());
            this->pointsUidList.append(pointData.uid);
            pointData.customerInfo = QString::fromStdString(point.customer_info());
            pointData.activationObjectName = QString::fromStdString(point.activation_object_name());
            pointData.activationObjectNotes = QString::fromStdString(point.activation_object_notes());
            pointData.expiryDate = QString::fromStdString(point.expiry_date());
            pointData.installType = point.install_type();
            pointData.updateMode = point.update_mode();
            pointData.serviceMode = point.service_mode();
            pointData.sumPoints = point.sum_points();
            pointData.remainingPoints = point.remaining_points();
            pointData.contractNo = QString::fromStdString(point.contract_no());
            pointData.projectName = QString::fromStdString(point.subject_name());
            pointData.enableRecycling = point.enable_recycling();
            pointData.warrantyDays = point.warranty_days();
            pointData.computeExpiryDayMode = point.compute_expiry_day_mode();
            pointData.warrantyMonths = point.warranty_months();
            pointData.warrantyYears = point.warranty_years();
            pointsData.append(pointData);
        }
        return true;
    }
    else
    {
        KLOG_WARNING() << "connect server failed!";
        return false;
    }
}

bool LicenseGrpc::getRecords(QList<Point> &pointsData, QList<Activation> &useageData)
{
    KLOG_WARNING() << "current thread id :" << QThread::currentThreadId();
    if (this->getPointsRecordsRsp(pointsData))
    {
        foreach (QString uid, pointsUidList)
        {
            QList<Activation> useages;
            if (getActivationRecords(useages, uid))
            {
                useageData.append(useages);
            }
            else
            {
                return false;
            }
        }
        return true;
    }
    else
    {
        return false;
    }
}

bool LicenseGrpc::getActivationRecords(QList<Activation> &useagesData, QString pointUid)
{
    grpc::ClientContext context;
    ::ActivationRecordReq req;
    req.set_service_uid(pointUid.toStdString());
    ::ActivationRecordsRsp rsp;
    auto channel = grpc::CreateChannel(QString("%1:7788").arg(serverIP).toStdString(), grpc::InsecureChannelCredentials());
    auto stub = License::NewStub(channel);

    auto status = stub->GetActivationRecords(&context, req, &rsp);
    if (status.ok())
    {
        for (int i = 0; i < rsp.activation_records_size(); i++)
        {
            Activation useageData;
            auto activationRecord = rsp.activation_records(i);
            useageData.serviceUid = QString::fromStdString(activationRecord.service_uid());
            useageData.activationObjectName = QString::fromStdString(activationRecord.activation_object_name());
            useageData.machineCode = QString::fromStdString(activationRecord.machine_code());
            useageData.activationCode = QString::fromStdString(activationRecord.activation_code());
            useageData.expiryDate = QString::fromStdString(activationRecord.expiry_date());
            useageData.activationTime = QString::fromStdString(activationRecord.activation_time());
            useageData.clientConnectionStatus = activationRecord.client_connection_status();
            useageData.clientIp = QString::fromStdString(activationRecord.client_ip());
            useagesData.append(useageData);
        }
        return true;
    }
    else
    {
        KLOG_WARNING() << "connect server failed!";
        return false;
    }
}

bool LicenseGrpc::getActivationRecordsByCondRsp(QString keywords, int pageSize,
                                                int startPageNo, int endPageNo,
                                                int useCached, QList<Activation> &useagesData,
                                                int &useCachedSize)
{
    KLOG_WARNING() << "current thread id :" << QThread::currentThreadId();
    grpc::ClientContext context;
    ::ActivationRecordsByCondReq req;
    ::ActivationRecordsByCondRsp rsp;
    auto channel = grpc::CreateChannel(QString("%1:7788").arg(serverIP).toStdString(), grpc::InsecureChannelCredentials());
    auto stub = License::NewStub(channel);

    req.set_keywords(keywords.toStdString());
    req.set_page_size(pageSize);
    req.set_start_page_no(startPageNo);
    req.set_end_page_no(endPageNo);
    req.set_use_cached(useCached);

    auto status = stub->GetActivationRecordsByCond(&context, req, &rsp);
    if (status.ok())
    {
        if (rsp.error().empty())
        {
            for (int i = 0; i < rsp.query_pages_activation_records_size(); i++)
            {
                Activation useageData;
                auto activationRecord = rsp.query_pages_activation_records(i);
                useageData.serviceUid = QString::fromStdString(activationRecord.service_uid());
                useageData.activationObjectName = QString::fromStdString(activationRecord.activation_object_name());
                useageData.machineCode = QString::fromStdString(activationRecord.machine_code());
                useageData.activationCode = QString::fromStdString(activationRecord.activation_code());
                useageData.expiryDate = QString::fromStdString(activationRecord.expiry_date());
                useageData.activationTime = QString::fromStdString(activationRecord.activation_time());
                useageData.clientConnectionStatus = activationRecord.client_connection_status();
                useageData.clientIp = QString::fromStdString(activationRecord.client_ip());
                useagesData.append(useageData);
            }
            useCachedSize = rsp.all_pages_activation_records();
            return true;
        }
        else
        {
            KLOG_WARNING() << QString::fromStdString(rsp.error());
            return false;
        }
    }
    else
    {
        KLOG_WARNING() << "connect server failed!";
        return false;
    }
}

QString LicenseGrpc::importPointsRecords(QString filePath)
{
    QFile file(filePath);
    file.open(QIODevice::ReadOnly);
    QDataStream inStream(&file);
    QByteArray data;
    char buf[64] = {0};
    while (!inStream.atEnd())
    {
        if (-1 == inStream.readRawData(buf, 64))
        {
            return "read file failed!";
        }
        data.append(buf, 64);
    }

    grpc::ClientContext context;
    ::ImportPointsReq req;
    req.set_points_records_bytes(data.toStdString());
    ::ImportPointsRsp rsp;
    auto channel = grpc::CreateChannel(QString("%1:7788").arg(serverIP).toStdString(), grpc::InsecureChannelCredentials());
    auto stub = License::NewStub(channel);

    auto status = stub->ImportPointsRecords(&context, req, &rsp);
    if (status.ok())
    {
        return QString().fromStdString(rsp.error());
    }
    else
    {
        KLOG_WARNING() << "connect server failed!";
        return "connect server failed!";
    }
}

void LicenseGrpc::onGetActivationRecordsByCond(QString filter, int pageSize,
                                               int startPageNo, int endPageNo,
                                               int useCached)
{
    this->useagesData.clear();
    int useCachedSize;
    bool res = this->getActivationRecordsByCondRsp(filter, pageSize,
                                                   startPageNo, endPageNo,
                                                   useCached, this->useagesData, useCachedSize);
    emit getActivationRecordsByCondRsp(this->useagesData, useCachedSize, res);
}

void LicenseGrpc::onGetPointsRecords()
{
    this->pointsData.clear();
    bool res = this->getPointsRecordsRsp(this->pointsData);

    emit getPointsRecordsRsp(this->pointsData, res);
}

void LicenseGrpc::onGetRecordsByUID(QString UID)
{
    this->useagesData.clear();
    bool res = this->getActivationRecords(this->useagesData, UID);
    emit getRecordsByUIDRsp(this->useagesData, res);
}

void LicenseGrpc::onGetRecords()
{
    this->pointsData.clear();
    this->useagesData.clear();
    bool res = this->getRecords(this->pointsData, this->useagesData);
    emit this->getRecordsRsp(this->useagesData, res);
}

void LicenseGrpc::onImportPointsRecords(QString path)
{
    QString error = this->importPointsRecords(path);
    emit this->importPointsRecordsRsp(error);
}
