#ifndef KSLSERVERGRPC_H
#define KSLSERVERGRPC_H

#include <grpc/grpc.h>
#include <grpcpp/client_context.h>
#include <grpcpp/create_channel.h>
#include <qt5-log-i.h>
#include <QDataStream>
#include <QFile>
#include <QObject>
#include <QThread>
#include <QTimer>
#include "license.grpc.pb.h"

typedef struct Point
{
    QString uid;
    // 客户信息
    QString customerInfo;
    // 激活对象名称
    QString activationObjectName;
    // 激活对象名称解释
    QString activationObjectNotes;
    // 质保期
    QString expiryDate;
    // 安装类型
    qint32 installType;
    // 更新模式
    qint32 updateMode;
    // 人工服务模式(5x8或7x24)
    qint32 serviceMode;
    // 总激活点数
    qint32 sumPoints;
    // 剩余激活点数
    qint32 remainingPoints;
    // 合同号
    QString contractNo;
    // 项目名称
    QString projectName;
    // 支持回滚
    bool enableRecycling;

    qint32 warrantyDays;

    qint32 computeExpiryDayMode;

    qint32 warrantyMonths;

    qint32 warrantyYears;
} Point;

typedef struct Activation
{
    QString serviceUid;
    QString activationObjectName;
    QString machineCode;
    QString activationCode;
    QString expiryDate;
    QString activationTime;
    quint32 clientConnectionStatus;
    QString clientIp;
} Activation;

class LicenseGrpc : public QObject
{
    Q_OBJECT
public:
    LicenseGrpc(QObject *parent = NULL);
    bool getRecords(QList<Point> &pointsData, QList<Activation> &useageData);
    QString importPointsRecords(QString filePath);
    bool getActivationRecordsByCondRsp(QString filter, int pageSize,
                                       int startPageNo, int endPageNo,
                                       int useCached, QList<Activation> &useagesData,
                                       int &useCachedSize);
    bool getPointsRecordsRsp(QList<Point> &pointsData);
    bool getActivationRecords(QList<Activation> &useageData, QString pointUid);

private:
    QTimer *updateTimer;
    QString serverIP;
    QStringList pointsUidList;
    QList<Activation> useagesData;
    QList<Point> pointsData;

signals:
    void getActivationRecordsByCondRsp(QList<Activation> useagesData,
                                       int useCachedSize, bool res);
    void getPointsRecordsRsp(QList<Point> pointsData, bool res);
    void importPointsRecordsRsp(QString error);
    void getRecordsRsp(QList<Activation> useagesData, bool res);
    void getRecordsByUIDRsp(QList<Activation> useagesData, bool res);
public slots:
    void onGetActivationRecordsByCond(QString filter, int pageSize,
                                      int startPageNo, int endPageNo,
                                      int useCached);
    void onGetPointsRecords();
    void onImportPointsRecords(QString path);
    void onGetRecords();
    void onGetRecordsByUID(QString UID);
};

#endif  // KSLSERVERGRPC_H
