#include "activation-info-table.h"
#include <QApplication>
#include <QDesktopWidget>
#include <QtMath>
#include "kiran-message-box.h"
#include "ks-license/license-i.h"
#include "server-export.h"
#include "ui_activation-info-table.h"

#define UPDATE_ACTIVATION_CACHE 0
#define USE_ACTIVATION_CACHE 1

enum ACtivationTableHoriz
{
    ActivationNumber,
    ActivationServiceUid,
    ActivationActivationObjectName,
    ActivationMachineCode,
    ActivationActivationCode,
    ActivationExpiryDate,
    ActivationActivationTime,
    ActivationClientConnnectionStatus,
    ActivationClientIp,
    ActivationMax
};

ActivationInfoTable::ActivationInfoTable(QWidget *parent) : QWidget(parent),
                                                            ui(new Ui::ActivationInfoTable)
{
    ui->setupUi(this);

    m_serverGrpc = LicenseGrpcThread::getLicenseGrpc();
    m_pageSelect = new PageSelect(1, this);
    m_tableSize = 25;
    m_serverExport = NULL;
    ui->horizontalLayout_page_select_icon->addWidget(m_pageSelect);

    connect(m_pageSelect,
            SIGNAL(updatePage(int)),
            this,
            SLOT(onUpdatePage(int)));
    connect(ui->lineEdit_search,
            SIGNAL(returnPressed()),
            this,
            SLOT(on_search()));
    connect(this,
            SIGNAL(getActivationRecordsByCond(QString, int, int, int, int)),
            m_serverGrpc,
            SLOT(onGetActivationRecordsByCond(QString, int, int, int, int)),
            Qt::QueuedConnection);

    connect(m_serverGrpc,
            SIGNAL(getActivationRecordsByCondRsp(QList<Activation>, int, bool)),
            this,
            SLOT(onGetRecordsByCond(QList<Activation>, int, bool)),
            Qt::BlockingQueuedConnection);

    connect(ui->lineEdit_search,
            SIGNAL(returnPressed()),
            this,
            SLOT(on_pushButton_search_clicked()));

    initUI();
}

ActivationInfoTable::~ActivationInfoTable()
{
    delete ui;
}

void ActivationInfoTable::initUI()
{
    //防止需求变更，显示首页注释
    // updatePageBase();

    initTable();
    ui->pushButton_search->setIcon(QIcon(":/ksl-search.svg"));
    ui->lineEdit_search->setPlaceholderText("Search...");
    ui->pushButton_refresh->setIcon(QIcon(":/ksl-refresh.svg"));
    ui->horizontalLayout_page_select_icon->setAlignment(Qt::AlignHCenter);
    ui->widget_search->setProperty("editfocus", false);
    ui->lineEdit_search->installEventFilter(this);
    ui->pushButton_search->installEventFilter(this);

    style()->polish(this);
}

void ActivationInfoTable::initTable()
{
    m_TableModel = new QStandardItemModel(this);
    ui->tableView->setModel(m_TableModel);
    m_TableModel->setHorizontalHeaderLabels(QStringList() << QObject::tr("Number")
                                                          << QObject::tr("ServiceUid")
                                                          << QObject::tr("ActiveObject")
                                                          << QObject::tr("MachineCode")
                                                          << QObject::tr("ActivationCode")
                                                          << QObject::tr("ExpiryDate")
                                                          << QObject::tr("ActiveTime")
                                                          << QObject::tr("AliveStatus")
                                                          << QObject::tr("ClientIp"));
    m_searchKeys << " " /*加空字符串是为了让搜索关键字列表匹配枚举UseageTableHoriz*/
                 << "service_uid"
                 << "activation_object_name"
                 << "machine_code"
                 << "activation_code"
                 << "expiry_date"
                 << "activation_time"
                 << "keep_alive_timestamp"
                 << "client_ip";
    initTableBase(ui->tableView, "activationsInfoHeader");
    updateTableHeader();
    on_pushButton_refresh_clicked();
}

void ActivationInfoTable::initTableBase(QTableView *tableView, const QString &tableName)
{
    tableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
    tableView->horizontalHeader()->setObjectName(tableName);
    tableView->verticalHeader()->hide();
    tableView->setAlternatingRowColors(true);
    tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    tableView->setShowGrid(false);
    tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    tableView->horizontalHeader()->setStretchLastSection(true);
    tableView->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft | Qt::AlignVCenter);
}

void ActivationInfoTable::updateTableHeader()
{
    int tableWidth = ui->tableView->width();
    int cellWidth = tableWidth / 17;
    ui->tableView->horizontalHeader()->resizeSection(ActivationNumber, 1 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationServiceUid, 1 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationActivationObjectName, 2 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationMachineCode, 3 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationActivationCode, 3 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationExpiryDate, 2 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationActivationTime, 2 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationClientConnnectionStatus, 1 * cellWidth);
    ui->tableView->horizontalHeader()->resizeSection(ActivationClientIp, 1 * cellWidth);
}

void ActivationInfoTable::onGetRecordsByCond(QList<Activation> activationData, int useCachedSize, bool res)
{
    if (res)
    {
        m_pageSelect->update(qCeil(static_cast<double>(useCachedSize) /
                                   static_cast<double>(m_tableSize)));
        updateTable(activationData);
    }
    else
    {
        KiranMessageBox::message(this,
                                 QObject::tr("warning"),
                                 QObject::tr("Unable connet to Server"),
                                 KiranMessageBox::Ok);
    }
}

void ActivationInfoTable::updateTable(QList<Activation> activationDataList)
{
    m_TableModel->removeRows(0, m_TableModel->rowCount());
    int row = m_tableSize * (m_pageSelect->getCurrentPage() - 1);
    for (int i = 0; i < activationDataList.length(); i++)
    {
        setTableItem(activationDataList[i], row++);
    }
}

void ActivationInfoTable::setTableItem(Activation activationData, int lineNo)
{
    QStringList dataStrList;
    int row = m_TableModel->rowCount();
    ui->tableView->setRowHeight(row, 36);

    dataStrList << activationData.serviceUid
                << activationData.activationObjectName
                << activationData.machineCode
                << activationData.activationCode
                << activationData.expiryDate
                << activationData.activationTime
                << QString((activationData.clientConnectionStatus == ClientConnectionStatus::CLIENT_CONNECTION_STATUS_ONLINE)
                               ? QObject::tr("online")
                               : QObject::tr("off-line"))
                << activationData.clientIp;
    m_TableModel->setItem(row, ActivationNumber, new QStandardItem(QString("%1").arg(lineNo + 1)));
    for (int i = 0; i < dataStrList.length(); i++)
    {
        if (dataStrList[i] == "")
        {
            dataStrList[i] = " - ";
        }
        m_TableModel->setItem(row, i + 1, new QStandardItem(dataStrList[i]));
    }
    //设置在线显示绿色，离线显示红色
    m_TableModel->item(row, ActivationClientConnnectionStatus)
        ->setForeground(QBrush((activationData.clientConnectionStatus == ClientConnectionStatus::CLIENT_CONNECTION_STATUS_ONLINE)
                                   ? QColor("#2ab270")
                                   : QBrush(QColor(255, 0, 0))));
}

void ActivationInfoTable::on_pushButton_export_clicked()
{
    if (m_serverExport == NULL)
    {
        ServerExport *serverExport;
        m_serverExport = new KiranTitlebarWindow(NULL);
        serverExport = new ServerExport(m_serverExport);
        connect(serverExport, &ServerExport::sigClose, m_serverExport, [&]()
                { m_serverExport->close(); });
        m_serverExport->setAttribute(Qt::WA_QuitOnClose, false);
        m_serverExport->installEventFilter(this);
        m_serverExport->setWindowModality(Qt::ApplicationModal);
        m_serverExport->raise();

        int screenNum = QApplication::desktop()->screenNumber(QCursor::pos());
        QRect screenGeometry = QApplication::desktop()->screenGeometry(screenNum);
        m_serverExport->move(screenGeometry.x() + (screenGeometry.width() - m_serverExport->width()) / 2,
                             screenGeometry.y() + (screenGeometry.height() - m_serverExport->height()) / 2);

        m_serverExport->setTitle(tr("Export"));
        m_serverExport->setWindowContentWidget(serverExport);
        m_serverExport->show();
    }
}

void ActivationInfoTable::on_comboBox_show_lines_activated(int index)
{
    int pageSizeOptions[] = {25, 50, 75, 100};
    if (index > sizeof(pageSizeOptions))
    {
        KLOG_WARNING() << "index over range!";
        m_tableSize = pageSizeOptions[0];
        return;
    }
    m_tableSize = pageSizeOptions[index];
    m_pageSelect->gotoFirstPage();
    emit getActivationRecordsByCond("", m_tableSize, 1, 1, USE_ACTIVATION_CACHE);
}

void ActivationInfoTable::resizeEvent(QResizeEvent *event)
{
    updateTableHeader();
    //放大缩小显示原页码第一行逻辑**需求可能变更，此处代码备用
    /*double oldCurrentPage = m_pageSelect->getCurrentPage();
        int currentPage = qCeil((((oldCurrentPage - 1) * m_useageTableSize) + 1) / m_useageTableSize);
        m_pageSelect->forceChangeCurrentPage(currentPage);
        emit getActivationRecordsByCond(ui->lineEdit_activations_search->text(),
                                              m_useageTableSize, currentPage, currentPage, USE_ACTIVATION_CACHE);*/

    int rightHorizontalSpacerWidth = (ui->widget_activations_page_select->width() - m_pageSelect->width()) / 2;
    ui->horizontalSpacer_page_select_right->changeSize(rightHorizontalSpacerWidth, 17);
    ui->horizontalSpacer_page_select_left->changeSize(rightHorizontalSpacerWidth - ui->comboBox_show_lines->width(), 17);
}

bool ActivationInfoTable::eventFilter(QObject *watched, QEvent *event)
{
    if (watched == ui->lineEdit_search || ui->pushButton_search)
    {
        if (event->type() == QEvent::FocusIn)
        {
            ui->widget_search->setProperty("editfocus", true);
            style()->unpolish(ui->widget_search);
            style()->polish(ui->widget_search);
            ui->widget_search->update();
        }
        if (event->type() == QEvent::FocusOut)
        {
            ui->widget_search->setProperty("editfocus", false);
            style()->unpolish(ui->widget_search);
            style()->polish(ui->widget_search);
            ui->widget_search->update();
        }
    }

    if (watched == m_serverExport && event->type() == QEvent::Close)
    {
        m_serverExport->deleteLater();
        m_serverExport = NULL;
    }
    return QWidget::eventFilter(watched, event);
}

void ActivationInfoTable::onActiveCurrent(QString serviceUid)
{
    if (serviceUid.isEmpty())
    {
        updateTableHeader();
    }
    else
    {
        emit getActivationRecordsByCond(getAccurateSearchKeywords(m_searchKeys[ActivationServiceUid], serviceUid),
                                        m_tableSize,
                                        1, 1, UPDATE_ACTIVATION_CACHE);
        m_pageSelect->gotoFirstPage();
    }
}

void ActivationInfoTable::on_pushButton_search_clicked()
{
    m_searchKeyword = ui->lineEdit_search->text();
    m_pageSelect->gotoFirstPage();
    emit getActivationRecordsByCond(ui->lineEdit_search->text(),
                                    m_tableSize, 1, 1, UPDATE_ACTIVATION_CACHE);
}

void ActivationInfoTable::onUpdatePage(int page)
{
    emit getActivationRecordsByCond(ui->lineEdit_search->text(),
                                    m_tableSize,
                                    page,
                                    page,
                                    USE_ACTIVATION_CACHE);
}

void ActivationInfoTable::on_pushButton_refresh_clicked()
{
    KLOG_WARNING() << "current thread id :" << QThread::currentThreadId();
    m_pageSelect->gotoFirstPage();
    emit getActivationRecordsByCond(ui->lineEdit_search->text(),
                                    m_tableSize,
                                    1, 1, UPDATE_ACTIVATION_CACHE);
}
