#ifndef ACTIVATIONINFOTABLE_H
#define ACTIVATIONINFOTABLE_H

#include <QStandardItemModel>
#include <QTableView>
#include <QWidget>
#include "grpc/license-grpc-thread.h"
#include "kiran-titlebar-window.h"
#include "page-select.h"

namespace Ui
{
class ActivationInfoTable;
}

class ActivationInfoTable : public QWidget
{
    Q_OBJECT
public:
    explicit ActivationInfoTable(QWidget *parent = NULL);
    ~ActivationInfoTable() override;

signals:
    void getActivationRecordsByCond(QString filter, int pageSize,
                                    int startPageNo, int endPageNo,
                                    int useCached);

public slots:
    virtual bool eventFilter(QObject *watched, QEvent *event) override;
    void onActiveCurrent(QString servieUid);

private slots:
    void on_comboBox_show_lines_activated(int index);
    void on_pushButton_search_clicked();
    void onUpdatePage(int page);
    void on_pushButton_export_clicked();
    void on_pushButton_refresh_clicked();
    void onGetRecordsByCond(QList<Activation> activationData, int useCachedSize, bool res);

protected:
    virtual void resizeEvent(QResizeEvent *event) override;

private:
    void initTable();
    void updateTable(QList<Activation> activationDataList);
    void setTableItem(Activation activationData, int lineNo);
    void initTableBase(QTableView *tableView, const QString &tableName);
    void updateTableHeader();
    void initUI();

    inline QString getAccurateSearchKeywords(QString key, QString value) { return "$" + key + "=" + value; }

private:
    LicenseGrpc *m_serverGrpc;
    PageSelect *m_pageSelect;
    int m_tableSize;
    QStandardItemModel *m_TableModel;
    QStringList m_searchKeys;
    KiranTitlebarWindow *m_serverExport;
    QString m_searchKeyword;

    Ui::ActivationInfoTable *ui;
};

#endif  // ACTIVATIONINFOTABLE_H
