#include "page-select.h"
#include "ui_page-select.h"

PageSelect::PageSelect(int totalPage, int maxShowPages, bool jumpEdit, QWidget *parent) : QWidget(parent),
                                                                                          ui(new Ui::PageSelect)
{
    ui->setupUi(this);
    m_totalPage = totalPage;
    m_maxShowPages = maxShowPages;
    m_currentPage = 1;
    m_pageSelectMode = PageSelectByButton;
    if (jumpEdit)
    {
        m_jumpLineEdit = new QLineEdit(this);
        m_jumpLineEdit->setMaximumSize(QSize(60, 30));
        m_jumpLineEdit->setMinimumSize(QSize(60, 30));
        ui->horizontalLayout_jump->addWidget(m_jumpLineEdit);
        m_jumpLineEdit->setValidator(new QIntValidator(this));
        m_jumpLineEdit->setPlaceholderText("Jump");
        connect(m_jumpLineEdit, SIGNAL(returnPressed()), this, SLOT(onJump()));
    }
    initUi();
    update(totalPage, m_currentPage);
}

PageSelect::PageSelect(int totalPage, QWidget *parent) : QWidget(parent),
                                                         ui(new Ui::PageSelect)
{
    ui->setupUi(this);
    m_totalPage = totalPage;
    ui->pushButton_page_last->hide();
    m_currentPage = 1;
    m_pageSelectMode = PageSelectByNo;
    ui->pushButton_left->setMaximumSize(80, 30);
    ui->pushButton_left->setMinimumSize(80, 30);
    ui->pushButton_left->setText(QObject::tr("PREV"));

    ui->pushButton_right->setMinimumSize(80, 30);
    ui->pushButton_right->setMaximumSize(80, 30);
    ui->pushButton_right->setText(QObject::tr("NEXT"));

    ui->horizontalLayout_apend_btn->addItem(new QSpacerItem(20, 30));

    m_jumpLineEdit = new QLineEdit(this);
    m_jumpLineEdit->setMaximumSize(QSize(50, 30));
    m_jumpLineEdit->setMinimumSize(QSize(50, 30));
    m_jumpLineEdit->setAlignment(Qt::AlignHCenter);
    ui->horizontalLayout_apend_btn->addWidget(m_jumpLineEdit);
    m_jumpLineEdit->setValidator(new QIntValidator(this));
    m_jumpLineEdit->setPlaceholderText("1");
    m_jumpLineEdit->setStyleSheet("QLineEdit{ border-width:1px;border-radius:2px;font-size:14px;\
                                        border:1px solid #393939;font-weight: 200;}"
                                  "QLineEdit:focus{border:1px solid #3EB3FF;}");

    QLabel *labelLine = new QLabel("-", this);
    labelLine->setMaximumSize(QSize(22, 30));
    labelLine->setMinimumSize(QSize(22, 30));
    labelLine->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    ui->horizontalLayout_apend_btn->addWidget(labelLine);

    m_totalPageLable = new QLabel(this);
    m_totalPageLable->setMaximumSize(QSize(50, 30));
    m_totalPageLable->setMinimumSize(QSize(50, 30));
    m_totalPageLable->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    m_totalPageLable->setStyleSheet("QLabel{background-color:#2d2d2d;border-radius:2px; "
                                    "color:#919191; border:1px solid #393939;font-size:14px;"
                                    "font-family: \"Noto Sans CJK SC regular\";font-weight: 200;}");
    ui->horizontalLayout_apend_btn->addWidget(m_totalPageLable);

    ui->horizontalLayout_apend_btn->addItem(new QSpacerItem(20, 30));

    updateTotalPageLable(totalPage);

    connect(m_jumpLineEdit, SIGNAL(returnPressed()), this, SLOT(onJump()));

    initUi();
    update(totalPage, m_currentPage);
    setWidth(336);
}

PageSelect::~PageSelect()
{
    delete ui;
}

void PageSelect::initUi()
{
    ui->pushButton_left->setStyleSheet("QPushButton{ background-color: #2d2d2d;outline: none;border:1px solid #393939;"
                                       "border-radius: 2px;font-weight: 200;} "
                                       "QPushButton:checked{ background-color: #2eb3ff;}"
                                       "QPushButton:hover{ background-color: #383838;}");

    ui->pushButton_right->setStyleSheet("QPushButton{ background-color: #2d2d2d;outline: none;border:1px solid #393939;"
                                        "border-radius: 2px;font-weight: 200;} "
                                        "QPushButton:checked{ background-color: #2eb3ff;}"
                                        "QPushButton:hover{ background-color: #383838;}");

    if (m_pageSelectMode == PageSelectByButton)
    {
        ui->pushButton_page_last->setCheckable(true);
        ui->pushButton_page_last->setAutoExclusive(true);
        ui->pushButton_page_last->setText(QString("%1").arg(m_totalPage));
        ui->pushButton_page_last->setStyleSheet("QPushButton{ background-color: #2d2d2d;border-width:1;border-radius: 0px;"
                                                "outline: none;font-weight: 200;} "
                                                "QPushButton:checked{ background-color: #2eb3ff;}");
        connect(ui->pushButton_page_last, SIGNAL(clicked()), this, SLOT(onSelectButtonClicked()));
    }
}

void PageSelect::onJump()
{
    update(m_totalPage, m_jumpLineEdit->text().toInt());
    emit updatePage(m_currentPage);
}

void PageSelect::onSelectButtonClicked()
{
    QPushButton *btn = qobject_cast<QPushButton *>(sender());
    if (btn->text().toInt() == m_currentPage)
    {
        return;
    }
    update(m_totalPage, btn->text().toInt());
    emit updatePage(m_currentPage);
}

void PageSelect::handleFirstLastPage(int currentPage)
{
    if (currentPage == 1 && currentPage != m_totalPage)
    {
        ui->pushButton_left->setEnabled(false);
        ui->pushButton_right->setEnabled(true);
    }
    else if (currentPage == m_totalPage && currentPage != 1)
    {
        ui->pushButton_right->setEnabled(false);
        ui->pushButton_left->setEnabled(true);
    }
    else if (currentPage == 1 && currentPage == m_totalPage)
    {
        ui->pushButton_right->setEnabled(false);
        ui->pushButton_left->setEnabled(false);
    }
    else
    {
        ui->pushButton_left->setEnabled(true);
        ui->pushButton_right->setEnabled(true);
    }
}

void PageSelect::emptyPageBtnList()
{
    while (!m_buttonList.isEmpty())
    {
        delete m_buttonList.first();
        m_buttonList.removeFirst();
    }
}

void PageSelect::createPageBtnList(int btnListSize)
{
    for (int i = 1; i < btnListSize; i++)
    {
        QPushButton *btn = new QPushButton(this);
        btn->setMaximumSize(QSize(30, 30));
        btn->setMinimumSize(QSize(30, 30));
        btn->setText(QString::number(i));
        btn->setCheckable(true);
        btn->setAutoExclusive(true);
        connect(btn, SIGNAL(clicked()), this, SLOT(onSelectButtonClicked()));
        btn->setStyleSheet("QPushButton{ background-color: #2d2d2d;border-width:1;border-radius: 0px;outline: none;} "
                           "QPushButton:checked{ background-color: #2eb3ff;}");
        ui->horizontalLayout_apend_btn->addWidget(btn);
        m_buttonList.append(btn);
    }
}

void PageSelect::update(int totalPage, int currentPage)
{
    if (currentPage > totalPage || currentPage <= 0)
    {
        KLOG_WARNING() << QString("current page overrange:current page = %1").arg(QString::number(currentPage));
        return;
    }
    ui->pushButton_page_last->setText(QString("%1").arg(totalPage));
    m_totalPage = totalPage;
    m_currentPage = currentPage;

    //首尾页判断
    handleFirstLastPage(currentPage);

    if (m_pageSelectMode == PageSelectByNo)
    {
        updateTotalPageLable(totalPage);
        m_jumpLineEdit->setText(QString::number(currentPage));
        return;
    }

    if (totalPage <= m_maxShowPages)
    {
        //清空链表重新插入
        emptyPageBtnList();
        createPageBtnList(totalPage);
        if (currentPage == totalPage)
        {
            ui->pushButton_page_last->setChecked(true);
        }
    }
    else
    {
        //创建按键
        if (m_buttonList.length() < m_maxShowPages - 1)
        {
            emptyPageBtnList();
            createPageBtnList(m_maxShowPages);
        }
        //按键逻辑
        if (currentPage <= m_maxShowPages - 3)
        {
            m_buttonList.last()->setText("...");
            m_buttonList.last()->setEnabled(false);
            m_buttonList.first()->setEnabled(true);
            for (int i = 0; i < m_maxShowPages - 2; i++)
            {
                m_buttonList[i]->setText(QString::number(i + 1));
            }
        }
        else if (currentPage >= totalPage - 3)
        {
            int pageNo = totalPage - 1;
            m_buttonList.first()->setText(("..."));
            m_buttonList.first()->setEnabled(false);
            m_buttonList.last()->setEnabled(true);
            for (auto iter = --m_buttonList.end(); iter != m_buttonList.begin(); iter--)
            {
                (*iter)->setText(QString::number(pageNo));
                pageNo--;
            }
        }
        else
        {
            int pageNo = currentPage + 1;
            m_buttonList.first()->setText(("..."));
            m_buttonList.first()->setEnabled(false);
            m_buttonList.last()->setText(("..."));
            m_buttonList.last()->setEnabled(false);
            for (int i = m_maxShowPages - 3; i >= 1; i--)
            {
                m_buttonList[i]->setText(QString::number(pageNo));
                pageNo--;
            }
        }
    }
    setBtnChecked(currentPage);
}

int PageSelect::getCurrentPage()
{
    return m_currentPage;
}

void PageSelect::gotoFirstPage()
{
    update(m_totalPage, 1);
}

void PageSelect::forceChangeCurrentPage(int currentPage)
{
    if (currentPage <= 0)
    {
        return;
    }
    m_currentPage = currentPage;
}

void PageSelect::update(int totalPage)
{
    update(totalPage, m_currentPage);
}

void PageSelect::setBtnChecked(int currentPage)
{
    if (currentPage == m_totalPage)
    {
        ui->pushButton_page_last->setChecked(true);
    }
    for (int i = 0; i < m_buttonList.length(); i++)
    {
        if (m_buttonList[i]->text().toInt() == currentPage)
        {
            m_buttonList[i]->setChecked(true);
        }
    }
}

void PageSelect::updateTotalPageLable(int totalPage)
{
    m_totalPageLable->setText(QString::number(totalPage));
}

void PageSelect::setWidth(int width)
{
    setMinimumWidth(width);
    setMaximumWidth(width);
}

void PageSelect::on_pushButton_left_clicked()
{
    update(m_totalPage, --m_currentPage);
    emit updatePage(m_currentPage);
}

void PageSelect::on_pushButton_right_clicked()
{
    update(m_totalPage, ++m_currentPage);
    emit updatePage(m_currentPage);
}
