#ifndef PAGESELECT_H
#define PAGESELECT_H

#include <kiran-log/qt5-log-i.h>
#include <QIntValidator>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>
#include <QWidget>

namespace Ui
{
class PageSelect;
}

class PageSelect : public QWidget
{
    Q_OBJECT

public:
    /**
      * @brief PageSelect:初始化页面选择界面，使用该函数初始化为多按钮选择
      * @param[in] totalPage   需要创建的页面选择界面控制的最大页数
      * @param[in] maxShowPages 必须大于等于5，建议最大不不超过8
      * @param[in] jumpEdit 是否需要页面跳转
      */
    explicit PageSelect(int totalPage, int maxShowPages, bool jumpEdit, QWidget *parent = NULL);
    /**
      * @brief PageSelect:初始化页面选择界面，使用该函数初始化为数字显示
      * @param[in] totalPage   需要创建的页面选择界面控制的最大页数
      */
    explicit PageSelect(int totalPage, QWidget *parent = NULL);
    ~PageSelect();

    int getCurrentPage();
    void gotoFirstPage();
    void update(int totalPage);

    void forceChangeCurrentPage(int currentPage);

private:
    enum PageSelectMode
    {
        PageSelectByButton,
        PageSelectByNo
    };

    Ui::PageSelect *ui;
    int m_totalPage, m_maxShowPages, m_currentPage;
    QList<QPushButton *> m_buttonList;
    QLineEdit *m_jumpLineEdit;
    QLabel *m_totalPageLable;
    int m_pageSelectMode;

    void initUi();
    /**
      * @param[in] totalPage   需要创建的页面选择界面控制的最大页数
      * @param[in] currentPage 必须大于等于1，不得超过totalPage
      */
    void update(int totalPage, int currentPage);
    void setBtnChecked(int currentPage);
    void updateTotalPageLable(int totalPage);
    void setWidth(int width);
    void handleFirstLastPage(int currentPage);
    void emptyPageBtnList();
    void createPageBtnList(int btnListSize);
private slots:
    void onJump();
    void onSelectButtonClicked();
    void on_pushButton_left_clicked();
    void on_pushButton_right_clicked();
signals:
    void updatePage(int page);
};

#endif  // PAGESELECT_H
