#include "server-export.h"
#include "ui_server-export.h"

ServerExport::ServerExport(QWidget *parent) : QWidget(parent),
                                              ui(new Ui::ServerExport)
{
    ui->setupUi(this);
    m_serverGrpc = LicenseGrpcThread::getLicenseGrpc();
    connect(this,
            SIGNAL(sigExportAll()),
            m_serverGrpc,
            SLOT(onGetRecords()),
            Qt::QueuedConnection);
    connect(this,
            SIGNAL(sigExportUID(QString)),
            m_serverGrpc,
            SLOT(onGetRecordsByUID(QString)),
            Qt::QueuedConnection);
    connect(m_serverGrpc,
            SIGNAL(getRecordsByUIDRsp(QList<Activation>, bool)),
            this,
            SLOT(onExport(QList<Activation>, bool)),
            Qt::BlockingQueuedConnection);
    connect(m_serverGrpc,
            SIGNAL(getRecordsRsp(QList<Activation>, bool)),
            this,
            SLOT(onExport(QList<Activation>, bool)),
            Qt::BlockingQueuedConnection);
    initUi();
}

ServerExport::~ServerExport()
{
    delete ui;
}

void ServerExport::initUi()
{
    QList<Point> points;
    m_serverGrpc->getPointsRecordsRsp(points);
    foreach (Point point, points)
    {
        ui->comboBoxUID->addItem(point.uid);
    }
    ui->comboBoxUID->setEnabled(false);
}

void ServerExport::on_pushButtonStartExport_clicked()
{
    if (ui->checkBoxExportAll->isChecked() || ui->checkBoxExportUID->isChecked())
    {
        m_csvFileName = QFileDialog::getSaveFileName(this,
                                                     tr("Save File"),
                                                     QDir::homePath(),
                                                     tr("(*.csv)"));
        if (m_csvFileName.isEmpty())
        {
            return;
        }

        if (!m_csvFileName.contains(".csv"))
        {
            m_csvFileName += ".csv";
        }

        if (ui->checkBoxExportAll->isChecked())
        {
            emit sigExportAll();
        }
        else if (ui->checkBoxExportUID->isChecked())
        {
            emit sigExportUID(ui->comboBoxUID->currentText());
        }
    }
    else
    {
        KiranMessageBox::message(this,
                                 QObject::tr("warning"),
                                 QString(tr("Please check export way!")),
                                 KiranMessageBox::Ok);
        emit sigClose();
    }
}

void ServerExport::on_checkBoxExportAll_stateChanged(int arg1)
{
    KLOG_WARNING() << "on_checkBoxExportAll_stateChanged" << arg1;
    if (arg1)
    {
        ui->checkBoxExportUID->setCheckState(Qt::Unchecked);
        ui->comboBoxUID->setEnabled(false);
    }
}

void ServerExport::on_checkBoxExportUID_stateChanged(int arg1)
{
    if (arg1)
    {
        ui->checkBoxExportAll->setCheckState(Qt::Unchecked);
        ui->comboBoxUID->setEnabled(true);
    }
    else
    {
        ui->comboBoxUID->setEnabled(false);
    }
}

void ServerExport::onExport(QList<Activation> useagesData, bool res)
{
    pullExportRes(res);
    if (!res)
    {
        KLOG_WARNING() << "Export UID Failed!";
        return;
    }

    QFile csv_file(m_csvFileName);
    csv_file.open(QIODevice::ReadWrite | QIODevice::Truncate);

    QString headLine(tr("Service UID, Object Name, Machine Code, Activation Code, Expiry Date, Activation Date, Client Ip\n"));
    csv_file.write(headLine.toUtf8());

    for (int i = 0; i < useagesData.length(); i++)
    {
        csv_file.write((useagesData[i].serviceUid + ", ").toUtf8());
        csv_file.write((useagesData[i].activationObjectName + ", ").toUtf8());
        csv_file.write((useagesData[i].machineCode + ", ").toUtf8());
        csv_file.write((useagesData[i].activationCode + ", ").toUtf8());
        csv_file.write((useagesData[i].expiryDate + ", ").toUtf8());
        csv_file.write((useagesData[i].activationTime + ", ").toUtf8());
        csv_file.write(useagesData[i].clientIp.toUtf8());
        csv_file.write("\n");
    }
}

void ServerExport::pullExportRes(bool res)
{
    if (!res)
    {
        KiranMessageBox::message(this,
                                 QObject::tr("warning"),
                                 QString(tr("Export Failed!")),
                                 KiranMessageBox::Ok);
        emit sigClose();
    }
    else
    {
        KiranMessageBox::message(this,
                                 QObject::tr("success"),
                                 QString(tr("Export success!")),
                                 KiranMessageBox::Ok);
        emit sigClose();
    }
}
