#include "server-guide.h"
#include <qt5-log-i.h>
#include <iostream>
#include "ui_server-guide.h"

ServerGuide::ServerGuide(QWidget *parent) : QWidget(parent),
                                            ui(new Ui::ServerGuide)
{
    ui->setupUi(this);
    // m_baseInfoItem = createActiveItem(QObject::tr("Base Information"), ":/ksl-base.svg");
    m_pointsManageItem = createActiveItem(ITEM_POINT_MANAGER, ":/ksl-points.svg");
    m_ActivationsInfoItem = createActiveItem(ITEM_ACTIVATIONS, ":/ksl-useage.svg");
    ui->listWidget->setSpacing(8);
    connect(ui->listWidget,
            SIGNAL(itemClicked(QListWidgetItem *)),
            this,
            SLOT(onItemClicked(QListWidgetItem *)));
    ui->listWidget->item(pointsManage)->setSelected(true);
    m_pointsManageItem->showArrow(true);
}

ServerGuide::~ServerGuide()
{
    delete ui;
}

ServerListItem *ServerGuide::createActiveItem(const QString &text, const QString iconPath)
{
    QListWidgetItem *newItem = new QListWidgetItem(ui->listWidget);
    ServerListItem *customItem = new ServerListItem(ui->listWidget);

    newItem->setSizeHint(QSize(190, 50));
    newItem->setTextAlignment(Qt::AlignVCenter);

    customItem->setItemText(text);
    customItem->setItemSvg(iconPath);
    ui->listWidget->addItem(newItem);
    ui->listWidget->setItemWidget(newItem, customItem);

    return customItem;
}

void ServerGuide::onItemClicked(QListWidgetItem *item)
{
    ServerListItem *customItem = dynamic_cast<ServerListItem *>(ui->listWidget->itemWidget(item));
    QString itemText;
    customItem->getItemText(itemText);
    /* if (itemText == QObject::tr("Base Information"))
    {
        emit itemClicked(ServerGuideWidget::baseInfo);
    }
    else */

    if (itemText == ITEM_ACTIVATIONS)
    {
        emit itemClicked(ServerGuide::activationsInfo);
        m_ActivationsInfoItem->showArrow(true);
        m_pointsManageItem->showArrow(false);
    }
    else if (itemText == ITEM_POINT_MANAGER)
    {
        emit itemClicked(ServerGuide::pointsManage);
        m_ActivationsInfoItem->showArrow(false);
        m_pointsManageItem->showArrow(true);
    }
    else
    {
        KLOG_WARNING() << "no such item";
    }
}

void ServerGuide::setUseagePointNumber(int pointsNumber)
{
    m_ActivationsInfoItem->setUseagesNumber(pointsNumber);
}

void ServerGuide::setCurrentItem(ServerGuide::ItemClickedType item)
{
    switch (item)
    {
    case ServerGuide::baseInfo:

    case ServerGuide::activationsInfo:
        emit itemClicked(activationsInfo);
        ui->listWidget->item(activationsInfo)->setSelected(true);
        m_ActivationsInfoItem->showArrow(true);
        m_pointsManageItem->showArrow(false);
        break;
    case ServerGuide::pointsManage:
        emit itemClicked(pointsManage);
        ui->listWidget->item(pointsManage)->setSelected(true);
        m_ActivationsInfoItem->showArrow(false);
        m_pointsManageItem->showArrow(true);
        break;
    }
}
