#ifndef ACTIVEGUIDE_H
#define ACTIVEGUIDE_H

#include <QWidget>
#include "cmake-define.h"
#include "dialog.h"
#include "ks-license/license-i.h"
#include "qrcode/show-qrcode.h"

namespace Ui
{
class ActiveGuide;
}

namespace VA
{
struct LicenseInfo
{
    LicenseInfo() : licensed(LicenseActivationStatus::LAS_LAST),
                    serviceStatus(LicenseServiceStatus::LICENSE_SERVICE_STATUS_LAST),
                    expiredTime(0),
                    activationType(LicenseActivationType::LICENSE_ACTIVATION_TYPE_UNKNOWN),
                    firstBootTime(0),
                    activationTime(0),
                    updateMode(LicenseUpdateMode::LICENSE_UPDATE_MODE_NONE),
                    serviceMode(LicenseServiceMode::LICENSE_SERVICE_MODE_NONE),
                    exceptionStatus(LicenseExceptionStatus::LICENSE_EXCEPTION_STATUS_NONE) {}
    // 激活状态
    qint32 licensed;
    // 服务状态
    qint32 serviceStatus;
    // 激活码
    QString activationCode;
    // 机器码
    QString machineCode;
    // 客户代码
    QString customerCode;
    // 过期时间
    time_t expiredTime;
    // 激活方式，如果为LICENSE_ACTIVATION_TYPE_UNKNOWN，则激活时间字段不可用
    qint32 activationType;
    // 第一次使用系统的时间
    time_t firstBootTime;
    // 激活时间
    time_t activationTime;
    // 激活服务器地址
    QString authServer;
    // 服务号
    QString serviceUid;
    // 更新模式
    qint32 updateMode;
    // 服务模式
    qint32 serviceMode;
    // 异常状态
    qint32 exceptionStatus;
};

class ActiveGuide : public QWidget
{
    Q_OBJECT

private:
    Ui::ActiveGuide *ui;
    QString m_object;
    ShowQRCode *m_showQRCode;
    bool m_reactiveFlag;
    VA::Dialog *m_dialog;
    LicenseInfo m_licenseInfo;

public:
    explicit ActiveGuide(const QString &object, const QIcon &icon, QWidget *parent = NULL);
    ~ActiveGuide();

protected:
    bool eventFilter(QObject *target, QEvent *e);

private:
    LicenseInfo getLicenseInfo();
    void initUi();
    void displayMessageBox(const QString &title, const QString &message);

signals:
    void sigClose();
    void sigAcived(time_t expiryDay);
    void sig_showQRCodeWgt(QPoint oPoint, QObject *target);

private slots:
    void on_pushButton_Active_clicked();
    void on_pushButton_Cancel_clicked();
    void popupQRCode(QPoint oPoint, QObject *target);
};
}  // namespace VA

#endif  // ACTIVEGUIDE_H
