#include "ksl-va.h"
#include <kiran-log/qt5-log-i.h>
#include <QCoreApplication>
#include <QDialog>
#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>
#include <QMessageBox>
#include <QVBoxLayout>
#include "dbus/license-dbus.h"

#define TRANSLATION_DIR TRANSLATIONS_FILE_DIR

KslVA::KslVA(const QString &ObjectName, const QIcon &icon, QWidget *parent) : QWidget(parent)
{
    ///加载翻译文件
    m_translator = new QTranslator();
    if (!m_translator->load(QLocale(),
                            "ksl-va",
                            "_",
                            TRANSLATION_DIR,
                            ".qm"))
    {
        KLOG_DEBUG() << "Kiran cpanel system load translation failed";
    }
    else
    {
        QCoreApplication::installTranslator(m_translator);
    }

    activeGuide = new VA::ActiveGuide(ObjectName, icon, parent);

    QFile file(":/ksl-va.qss");
    if (file.open(QFile::ReadOnly))
    {
        QString styleSheet = QLatin1String(file.readAll());

        activeGuide->setStyleSheet(styleSheet);
        file.close();
    }
    else
    {
        QMessageBox::warning(this, tr("Warning"), tr("load qss file failed"), QMessageBox::Ok);
    }

    connect(activeGuide, SIGNAL(sigClose()), this, SLOT(onSigCLose()));
    connect(activeGuide, SIGNAL(sigAcived(time_t)), this, SLOT(on_sigAcived(time_t)));

    frameGeometry().setHeight(48);
    setWindowFlags(windowFlags() & ~Qt::WindowMaximizeButtonHint);

    auto layout = new QVBoxLayout(this);
    layout->addWidget(activeGuide);
    setWindowIcon(icon);
    setWindowTitle(tr("Sorftware Activation"));
    setStyleSheet("background-color: #2D2D2D");
}

static LicenseActivationStatus getLicenseState(const QString &ObjectName, QString &error)
{
    QString licenseStr;
    if (!InfoDbus::KylinLicense::getLicenseJson(licenseStr, ObjectName))
    {
        error = QObject::tr("can't connect to DBUS interface");
        return LicenseActivationStatus::LAS_LAST;
    }

    QJsonParseError jsonError;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(licenseStr.toLocal8Bit().data(), &jsonError);
    if (jsonDocument.isNull() || jsonError.error != QJsonParseError::NoError)
    {
        error = QObject::tr("the license JSon String Error");
        return LicenseActivationStatus::LAS_LAST;
    }
    if (jsonDocument.isObject())
    {
        QJsonObject obj = jsonDocument.object();
        if (obj.contains(LICENSE_JK_ACTIVATION_STATUS))
        {
            QJsonValue value = obj.take(LICENSE_JK_ACTIVATION_STATUS);
            if (value.isDouble())
            {
                return (LicenseActivationStatus)value.toVariant().toInt();
            }
        }
        else
        {
            error = QObject::tr("No such object named:{1}").arg(LICENSE_JK_ACTIVATION_STATUS);
            return LicenseActivationStatus::LAS_LAST;
        }
    }

    error = QObject::tr("Unkonw error");
    return LicenseActivationStatus::LAS_LAST;
}

time_t getExpiryTime(const QString &ObjectName, QString &error)
{
    QString licenseStr;
    if (!InfoDbus::KylinLicense::getLicenseJson(licenseStr, ObjectName))
    {
        error = QObject::tr("can't connect to DBUS interface");
        return 0;
    }

    QJsonParseError jsonError;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(licenseStr.toLocal8Bit().data(), &jsonError);
    if (jsonDocument.isNull() || jsonError.error != QJsonParseError::NoError)
    {
        error = QObject::tr("the license JSon String Error");
        return 0;
    }
    if (jsonDocument.isObject())
    {
        QJsonObject obj = jsonDocument.object();
        if (obj.contains(LICENSE_JK_EXPIRED_TIME))
        {
            QJsonValue value = obj.take(LICENSE_JK_EXPIRED_TIME);
            if (value.isDouble())
            {
                return (time_t)value.toVariant().toInt();
            }
        }
        else
        {
            error = QObject::tr("No such object named:{1}").arg(LICENSE_JK_EXPIRED_TIME);
            return LicenseActivationStatus::LAS_LAST;
        }
    }

    error = QObject::tr("Unkonw error");
    return LicenseActivationStatus::LAS_LAST;
}

void KslVA::on_sigAcived(time_t expiryDay)
{
    emit sigActived(expiryDay);
}

void KslVA::onSigCLose()
{
    close();
}
